/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: StatefulSessionFactory.java 9 2006-02-19 18:53:32Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.container.session.stateful;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

import org.objectweb.easybeans.api.EZBContainer;
import org.objectweb.easybeans.api.FactoryException;
import org.objectweb.easybeans.api.bean.EasyBeansSFSB;
import org.objectweb.easybeans.api.bean.injection.EasyBeansInjectionException;
import org.objectweb.easybeans.container.session.SessionFactory;
import org.objectweb.easybeans.log.JLog;
import org.objectweb.easybeans.log.JLogFactory;
import org.objectweb.easybeans.pool.JPool;
import org.objectweb.easybeans.pool.PoolEntryStatistics;
import org.objectweb.easybeans.pool.PoolException;
import org.objectweb.easybeans.pool.PoolFactory;
import org.objectweb.easybeans.rpc.JEJBResponse;
import org.objectweb.easybeans.rpc.api.EJBRequest;
import org.objectweb.easybeans.rpc.api.EJBResponse;
import org.objectweb.easybeans.rpc.api.RPCException;

/**
 * This class manages the stateless session bean and its creation/lifecycle.
 * @author Florent Benoit
 */
public class StatefulSessionFactory extends SessionFactory<EasyBeansSFSB> implements
        PoolFactory<EasyBeansSFSB, Long> {

    /**
     * Logger.
     */
    private static JLog logger = JLogFactory.getLog(StatefulSessionFactory.class);

    /**
     * Id generator.
     */
    private Long idCount = Long.valueOf(0L);

    /**
     * Builds a new factory with a given name and its container.
     * @param className name of this factory (name of class that is managed)
     * @param container the root component of this factory.
     * @throws FactoryException if class can't be loaded.
     */
    public StatefulSessionFactory(final String className, final EZBContainer container) throws FactoryException {
        super(className, container);
        setPool(new JPool<EasyBeansSFSB, Long>(this));
    }

    /**
     * Creates an instance with the given hint.
     * @param clue a clue given by the Pool. Could be null.
     * @throws PoolException if instance cannot be created.
     * @return the created instance.
     */
    public EasyBeansSFSB create(final Long clue) throws PoolException {
        EasyBeansSFSB instance = null;
        try {
            instance = getBeanClass().newInstance();
        } catch (InstantiationException e) {
            throw new PoolException("Cannot create a new instance", e);
        } catch (IllegalAccessException e) {
            throw new PoolException("Cannot create a new instance", e);
        }

        // Set the factory
        instance.setEasyBeansFactory(this);


        // call dependency injection
        try {
            instance.injectedByEasyBeans();
        } catch (EasyBeansInjectionException e) {
            throw new PoolException("Cannot inject resources in the created bean", e);
        }

        // post construct callback
        instance.postConstructEasyBeansLifeCycle();

        return instance;
    }

    /**
     * Checks if the given object with the given clue is matching.
     * @param bean given object against which the check should be done.
     * @param clue the object used as clue to check the matching.
     * @return true if it is matching, else false.
     */
    public boolean isMatching(final EasyBeansSFSB bean, final Long clue) {
        logger.debug("Called with bean = {0} and beanId = {1}", bean, clue);
        Long beanId = idMap.get(bean);
        boolean val = (beanId.equals(clue));
        logger.debug("Found id = {0} and will return = {1}", beanId, Boolean.valueOf(val));

        return val;
    }

    /**
     * Validate an instance by giving some statistics.
     * @param object the instance to validate
     * @param stats some statistics to help in the validating process.
     * @return true if the element is valid, else false.
     */
    public boolean validate(final EasyBeansSFSB object, final PoolEntryStatistics stats) {
        return true;
    }

    /**
     * Callback called when object is gonna be removed.
     * @param instance that is being removed from the pool.
     */
    public void remove(final EasyBeansSFSB instance) {
        // call callbacks
        instance.preDestroyEasyBeansLifeCycle();
    }

    /**
     * TODO: to be removed.
     * Map between bean and its id
     */
    private Map<EasyBeansSFSB, Long> idMap = new HashMap<EasyBeansSFSB, Long>();


    /**
     * A request comes to the bean factory and needs to be handled.<br>
     * A response is done which contains the answer.
     * @param request the EJB request.
     * @throws RPCException if the invocation fails.
     * @return a response that have been processed by the factory.
     */
    @Override
    public EJBResponse rpcInvoke(final EJBRequest request) throws RPCException {

        EasyBeansSFSB bean = null;
        Long beanId = request.getBeanId();
        logger.debug("Receive invocation with beanId = {0}", beanId);

        // no Id, compute a new one
        if (beanId == null) {
            idCount = Long.valueOf(idCount.longValue() + 1);
            beanId = idCount;
        }

        try {
                bean = getPool().get(beanId);
        } catch (PoolException e) {
            throw new RPCException("Cannot get element in the pool", e);
        }

        //TODO : remove this temp hack. id should be in the bean itself.

        logger.debug("Add in the map bean {0} with beanId = {1}", bean, beanId);
        idMap.put(bean, beanId);

        // get hash of the request
        long hash = request.getMethodHash();

        Method m = getHashes().get(Long.valueOf(hash));

        if (m == null) {
            throw new RPCException("Cannot find method called on the bean '" + getClassName() + "'.");
        }

        Object value = null;
        try {
            value = m.invoke(bean, request.getMethodArgs());
        } catch (IllegalArgumentException e) {
            throw new RPCException(e);
        } catch (IllegalAccessException e) {
            throw new RPCException(e);
        } catch (InvocationTargetException e) {
            throw new RPCException(e.getCause());
        } finally {
            // push back into the pool
            try {
                getPool().release(bean);
            } catch (PoolException e) {
                throw new RPCException("cannot release bean", e);
            }
        }

        EJBResponse ejbResponse = new JEJBResponse();
        ejbResponse.setValue(value);
        ejbResponse.setBeanId(beanId);
        return ejbResponse;
    }

}
