/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JavaxAnnotationEJBVisitor.java 9 2006-02-19 18:53:32Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.deployment.annotations.analyzer;

import org.objectweb.asm.Type;

import org.objectweb.easybeans.deployment.annotations.impl.JAnnotationEJB;
import org.objectweb.easybeans.deployment.annotations.metadata.interfaces.IAnnotationEJB;

/**
 * This class manages the handling of &#64;{@link javax.annotation.EJB} annotation.
 * @param <T> An implementation of IAnnotationEJB interface.
 * @author Florent Benoit
 */
public class JavaxAnnotationEJBVisitor<T extends IAnnotationEJB>
          extends AbsAnnotationVisitor<T> implements AnnotationType {

    /**
     * Name attribute of the annotation.
     */
    private static final String NAME = "name";

    /**
     * Bean interface attribute of the annotation.
     */
    private static final String BEAN_INTERFACE = "beanInterface";

    /**
     * Bean name attribute of the annotation.
     */
    private static final String BEAN_NAME = "beanName";

    /**
     * Type of annotation.
     */
    public static final String TYPE = "Ljavax/annotation/EJB;";

    /**
     * Internal object used representing &#64;{@link javax.annotation.EJB} annotation.
     */
    private JAnnotationEJB jAnnotationEJB = null;

    /**
     * Constructor.
     * @param annotationMetadata linked to a class or method metadata
     */
    public JavaxAnnotationEJBVisitor(final T annotationMetadata) {
        super(annotationMetadata);
        jAnnotationEJB = new JAnnotationEJB();
    }


    /**
     * Visits a primitive value of the annotation.<br>
     * @param name the value name.
     * @param value the actual value, whose type must be {@link Byte},
     *        {@link Boolean}, {@link Character}, {@link Short},
     *        {@link Integer}, {@link Long}, {@link Float}, {@link Double},
     *        {@link String} or {@link org.objectweb.asm.Type}.
     */
    @Override
    public void visit(final String name, final Object value) {
        if (name.equals(NAME)) {
            jAnnotationEJB.setName((String) value);
        } else if (name.equals(BEAN_INTERFACE)) {
            Type type = (Type) value;
            jAnnotationEJB.setBeanInterface(type.getClassName());
        } else if (name.equals(BEAN_NAME)) {
            jAnnotationEJB.setBeanName((String) value);
        }
    }

    /**
     * Visits the end of the annotation. <br>
     * Creates the object and store it.
     */
    @Override
    public void visitEnd() {
        // add object
        getAnnotationMetadata().setJAnnotationEJB(jAnnotationEJB);
    }

    /**
     * @return type of the annotation (its description)
     */
    public String getType() {
        return TYPE;
    }


    /**
     * @return Internal object used representing &#64;{@link javax.annotation.EJB} annotation.
     */
    protected JAnnotationEJB getJAnnotationEJB() {
        return jAnnotationEJB;
    }

    /**
     * Sets the jAnnotationEJB object.
     * @param jAnnotationEJB the object which replaced the previous one.
     */
    protected void setJAnnotationEJB(final JAnnotationEJB jAnnotationEJB) {
        this.jAnnotationEJB = jAnnotationEJB;
    }

}
