/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ScanClassVisitor.java 9 2006-02-19 18:53:32Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.deployment.annotations.analyzer;

import java.util.HashMap;
import java.util.Map;

import org.objectweb.asm.AnnotationVisitor;
import org.objectweb.asm.ClassVisitor;
import org.objectweb.asm.FieldVisitor;
import org.objectweb.asm.MethodVisitor;
import org.objectweb.asm.commons.EmptyVisitor;

import org.objectweb.easybeans.deployment.annotations.JField;
import org.objectweb.easybeans.deployment.annotations.JMethod;
import org.objectweb.easybeans.deployment.annotations.analyzer.classes.JavaxAnnotationEJBsVisitor;
import org.objectweb.easybeans.deployment.annotations.analyzer.classes.JavaxAnnotationResourcesVisitor;
import org.objectweb.easybeans.deployment.annotations.analyzer.classes.JavaxEjbApplicationExceptionVisitor;
import org.objectweb.easybeans.deployment.annotations.analyzer.classes.JavaxEjbLocalHomeVisitor;
import org.objectweb.easybeans.deployment.annotations.analyzer.classes.JavaxEjbLocalVisitor;
import org.objectweb.easybeans.deployment.annotations.analyzer.classes.JavaxEjbMessageDrivenVisitor;
import org.objectweb.easybeans.deployment.annotations.analyzer.classes.JavaxEjbRemoteHomeVisitor;
import org.objectweb.easybeans.deployment.annotations.analyzer.classes.JavaxEjbRemoteVisitor;
import org.objectweb.easybeans.deployment.annotations.analyzer.classes.JavaxEjbStatefulVisitor;
import org.objectweb.easybeans.deployment.annotations.analyzer.classes.JavaxEjbStatelessVisitor;
import org.objectweb.easybeans.deployment.annotations.analyzer.classes.JavaxEjbTransactionManagementVisitor;
import org.objectweb.easybeans.deployment.annotations.metadata.ClassAnnotationMetadata;
import org.objectweb.easybeans.deployment.annotations.metadata.EjbJarAnnotationMetadata;

/**
 * This classes analyses a given class and build/fill meta data information.
 * @author Florent Benoit
 */
public class ScanClassVisitor extends EmptyVisitor implements ClassVisitor {

    /**
     * Annotation visitor which do nothing.
     */
    private EmptyVisitor emptyVisitor = null;

    /**
     * Class generated by the visitor which correspond to meta data contained in.
     * the parsed class
     */
    private ClassAnnotationMetadata classAnnotationMetadata = null;

    /**
     * Map of annotation visitors used by this visitor.<br>
     * The key is the annotation descriptor (asm)
     */
    private Map<String, AnnotationVisitor> classAnnotationVisitors = null;

    /**
     * Parent of classannotation meta data that are built by this visitor.
     */
    private EjbJarAnnotationMetadata ejbJarAnnotationMetadata = null;

    /**
     * Constructor.
     * @param ejbJarAnnotationMetadata the parent object on which add generated
     *        meta-data
     */
    public ScanClassVisitor(final EjbJarAnnotationMetadata ejbJarAnnotationMetadata) {
        this.ejbJarAnnotationMetadata = ejbJarAnnotationMetadata;
        classAnnotationVisitors = new HashMap<String, AnnotationVisitor>();
    }

    /**
     * Build the meta-data class with the given name (given by asm).
     * @param className name of the class that is visited
     * @param superName the internal of name of the super class (see
     *        {@link org.objectweb.asm.Type#getInternalName() getInternalName}). For interfaces,
     *        the super class is {@link Object}. May be <tt>null</tt>, but
     *        only for the {@link Object} class.
     * @param interfaces the internal names of the class's interfaces (see
     *        {@link org.objectweb.asm.Type#getInternalName() getInternalName}). May be
     *        <tt>null</tt>.
     */
    private void init(final String className, final String superName, final String[] interfaces) {
        classAnnotationMetadata = new ClassAnnotationMetadata(className, ejbJarAnnotationMetadata);
        classAnnotationMetadata.setSuperName(superName);
        classAnnotationMetadata.setInterfaces(interfaces);
        initVisitors();
    }

    /**
     * Build visitors used by this one.
     */
    private void initVisitors() {
        emptyVisitor = new EmptyVisitor();

        // add @Local
        classAnnotationVisitors.put(JavaxEjbLocalVisitor.TYPE, new JavaxEjbLocalVisitor(classAnnotationMetadata));

        // add @Remote
        classAnnotationVisitors.put(JavaxEjbRemoteVisitor.TYPE, new JavaxEjbRemoteVisitor(classAnnotationMetadata));

        // add @Stateless
        classAnnotationVisitors.put(JavaxEjbStatelessVisitor.TYPE,
                new JavaxEjbStatelessVisitor(classAnnotationMetadata));

        // add @Stateful
        classAnnotationVisitors.put(JavaxEjbStatefulVisitor.TYPE, new JavaxEjbStatefulVisitor(classAnnotationMetadata));

        // add @MessageDriven
        classAnnotationVisitors.put(JavaxEjbMessageDrivenVisitor.TYPE, new JavaxEjbMessageDrivenVisitor(
                classAnnotationMetadata));

        // add @LocalHome
        classAnnotationVisitors.put(JavaxEjbLocalHomeVisitor.TYPE,
                new JavaxEjbLocalHomeVisitor(classAnnotationMetadata));

        // add @RemoteHome
        classAnnotationVisitors.put(JavaxEjbRemoteHomeVisitor.TYPE, new JavaxEjbRemoteHomeVisitor(
                classAnnotationMetadata));

        // add @TransactionManagement
        classAnnotationVisitors.put(JavaxEjbTransactionManagementVisitor.TYPE,
                new JavaxEjbTransactionManagementVisitor(classAnnotationMetadata));

        // add @TransactionAttribute
        classAnnotationVisitors.put(JavaxEjbTransactionAttributeVisitor.TYPE,
                new JavaxEjbTransactionAttributeVisitor<ClassAnnotationMetadata>(classAnnotationMetadata));

        // add @Interceptors
        classAnnotationVisitors.put(JavaxEjbInterceptorsVisitor.TYPE,
                new JavaxEjbInterceptorsVisitor<ClassAnnotationMetadata>(classAnnotationMetadata));

        // add @ApplicationException
        classAnnotationVisitors.put(JavaxEjbApplicationExceptionVisitor.TYPE, new JavaxEjbApplicationExceptionVisitor(
                classAnnotationMetadata));

        // add @EJB
        classAnnotationVisitors.put(JavaxAnnotationEJBVisitor.TYPE,
                new JavaxAnnotationEJBVisitor<ClassAnnotationMetadata>(classAnnotationMetadata));

        // add @EJBs
        classAnnotationVisitors.put(JavaxAnnotationEJBsVisitor.TYPE,
                new JavaxAnnotationEJBsVisitor(classAnnotationMetadata));

        // add @Resource
        classAnnotationVisitors.put(JavaxAnnotationResourceVisitor.TYPE,
                new JavaxAnnotationResourceVisitor<ClassAnnotationMetadata>(classAnnotationMetadata));

        // add @Resources
        classAnnotationVisitors.put(JavaxAnnotationResourcesVisitor.TYPE,
                new JavaxAnnotationResourcesVisitor(classAnnotationMetadata));


    }

    /**
     * Visits the header of the class.
     * @param version the class version.
     * @param access the class's access flags (see {@link org.objectweb.asm.Opcodes}). This
     *        parameter also indicates if the class is deprecated.
     * @param name the internal name of the class (see
     *        {@link org.objectweb.asm.Type#getInternalName() getInternalName}).
     * @param signature the signature of this class. May be <tt>null</tt> if
     *        the class is not a generic one, and does not extend or implement
     *        generic classes or interfaces.
     * @param superName the internal of name of the super class (see
     *        {@link org.objectweb.asm.Type#getInternalName() getInternalName}). For interfaces,
     *        the super class is {@link Object}. May be <tt>null</tt>, but
     *        only for the {@link Object} class.
     * @param interfaces the internal names of the class's interfaces (see
     *        {@link org.objectweb.asm.Type#getInternalName() getInternalName}). May be
     *        <tt>null</tt>.
     */
    @Override
    public void visit(final int version,
                      final int access,
                      final String name,
                      final String signature,
                      final String superName,
                      final String[] interfaces) {
        init(name, superName, interfaces);
    }

    /**
     * Visits an annotation of the class.
     * @param desc the class descriptor of the annotation class.
     * @param visible <tt>true</tt> if the annotation is visible at runtime.
     * @return a non null visitor to visit the annotation values.
     */
    @Override
    public AnnotationVisitor visitAnnotation(final String desc, final boolean visible) {
        AnnotationVisitor av = classAnnotationVisitors.get(desc);
        if (av != null) {
            return av;
        }
        return emptyVisitor;
    }

    /**
     * Visits a method of the class. This method <i>must</i> return a new
     * {@link org.objectweb.asm.MethodVisitor} instance (or <tt>null</tt>) each time it is
     * called, i.e., it should not return a previously returned visitor.
     * @param access the method's access flags (see {@link org.objectweb.asm.Opcodes}). This
     *        parameter also indicates if the method is synthetic and/or
     *        deprecated.
     * @param name the method's name.
     * @param desc the method's descriptor (see {@link org.objectweb.asm.Type}).
     * @param signature the method's signature. May be <tt>null</tt> if the
     *        method parameters, return type and exceptions do not use generic
     *        types.
     * @param exceptions the internal names of the method's exception classes
     *        (see {@link org.objectweb.asm.Type#getInternalName() getInternalName}). May be
     *        <tt>null</tt>.
     * @return an object to visit the byte code of the method, or <tt>null</tt>
     *         if this class visitor is not interested in visiting the code of
     *         this method.
     */
    @Override
    public MethodVisitor visitMethod(final int access, final String name, final String desc, final String signature,
            final String[] exceptions) {
        JMethod jMethod = new JMethod(access, name, desc, signature, exceptions);
        return new ScanMethodVisitor(jMethod, classAnnotationMetadata);
    }

    /**
     * Visits the end of the class. This method, which is the last one to be
     * called, is used to inform the visitor that all the fields and methods of
     * the class have been visited.
     */
    @Override
    public void visitEnd() {
        ejbJarAnnotationMetadata.addClassAnnotationMetadata(classAnnotationMetadata);
    }

    /**
     * Visits a field of the class.
     * @param access the field's access flags (see {@link org.objectweb.asm.Opcodes}). This
     *        parameter also indicates if the field is synthetic and/or
     *        deprecated.
     * @param name the field's name.
     * @param desc the field's descriptor (see {@link org.objectweb.asm.Type}).
     * @param signature the field's signature. May be <tt>null</tt> if the
     *        field's type does not use generic types.
     * @param value the field's initial value. This parameter, which may be
     *        <tt>null</tt> if the field does not have an initial value, must
     *        be an {@link Integer}, a {@link Float}, a {@link Long}, a
     *        {@link Double} or a {@link String} (for <tt>int</tt>,
     *        <tt>float</tt>, <tt>long</tt> or <tt>String</tt> fields
     *        respectively). <i>This parameter is only used for static fields</i>.
     *        Its value is ignored for non static fields, which must be
     *        initialized through bytecode instructions in constructors or
     *        methods.
     * @return a visitor to visit field annotations and attributes, or
     *         <tt>null</tt> if this class visitor is not interested in
     *         visiting these annotations and attributes.
     */
    @Override
    public FieldVisitor visitField(final int access, final String name, final String desc, final String signature,
            final Object value) {
        JField jField = new JField(access, name, desc, signature, value);
        return new ScanFieldVisitor(jField, classAnnotationMetadata);
    }
}
