/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ScanFieldVisitor.java 9 2006-02-19 18:53:32Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.deployment.annotations.analyzer;

import java.util.HashMap;
import java.util.Map;

import org.objectweb.asm.AnnotationVisitor;
import org.objectweb.asm.FieldVisitor;
import org.objectweb.asm.commons.EmptyVisitor;

import org.objectweb.easybeans.deployment.annotations.JField;
import org.objectweb.easybeans.deployment.annotations.analyzer.field.JavaxPersistencePersistenceContextVisitor;
import org.objectweb.easybeans.deployment.annotations.metadata.ClassAnnotationMetadata;
import org.objectweb.easybeans.deployment.annotations.metadata.FieldAnnotationMetadata;

/**
 * This classes analyses a given field and build/fill meta data information.
 * @author Florent Benoit
 */
public class ScanFieldVisitor extends EmptyVisitor implements FieldVisitor {

    /**
     * Annotation visitor which do nothing.
     */
    private EmptyVisitor emptyVisitor = null;

    /**
     * Class generated by the visitor which correspond to meta data contained in the parsed field.
     */
    private FieldAnnotationMetadata fieldAnnotationMetadata = null;

    /**
     * Map of annotation visitors used by this visitor.<br>
     * The key is the annotation descriptor (asm)
     */
    private Map<String, AnnotationVisitor> fieldAnnotationVisitors = null;

    /**
     * Parent of field annotation meta data that are built by this visitor.
     */
    private ClassAnnotationMetadata classAnnotationMetadata = null;

    /**
     * Constructor.
     * @param jField field object on which we set meta data.
     * @param classAnnotationMetadata the parent object on which add generated meta-data.
     */
    public ScanFieldVisitor(final JField jField, final ClassAnnotationMetadata classAnnotationMetadata) {

        // object build and to fill
        this.fieldAnnotationMetadata = new FieldAnnotationMetadata(jField, classAnnotationMetadata);

        // parent
        this.classAnnotationMetadata = classAnnotationMetadata;

        // list of visitors to use
        fieldAnnotationVisitors = new HashMap<String, AnnotationVisitor>();
        initVisitors();
    }

    /**
     * Build visitors used by this one.
     */
    private void initVisitors() {
        emptyVisitor = new EmptyVisitor();

        // add @EJB
        fieldAnnotationVisitors.put(JavaxAnnotationEJBVisitor.TYPE,
                new JavaxAnnotationEJBVisitor<FieldAnnotationMetadata>(fieldAnnotationMetadata));

        // add @Resource
        fieldAnnotationVisitors.put(JavaxAnnotationResourceVisitor.TYPE,
                new JavaxAnnotationResourceVisitor<FieldAnnotationMetadata>(fieldAnnotationMetadata));


        // add @PersistenceContext
        fieldAnnotationVisitors.put(JavaxPersistencePersistenceContextVisitor.TYPE,
                new JavaxPersistencePersistenceContextVisitor(fieldAnnotationMetadata));

    }

    /**
     * Visits an annotation of the field.
     * @param desc the class descriptor of the annotation class.
     * @param visible <tt>true</tt> if the annotation is visible at runtime.
     * @return a non null visitor to visit the annotation values.
     */
    @Override
    public AnnotationVisitor visitAnnotation(final String desc, final boolean visible) {
        AnnotationVisitor av = fieldAnnotationVisitors.get(desc);
        if (av != null) {
            return av;
        }
        return emptyVisitor;
    }

    /**
     * Visits the end of the method. This method, which is the last one to be
     * called, is used to inform the visitor that all the annotations and
     * attributes of the method have been visited.
     */
    @Override
    public void visitEnd() {
        classAnnotationMetadata.addFieldAnnotationMetadata(fieldAnnotationMetadata);
    }

}
