/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ScanMethodVisitor.java 9 2006-02-19 18:53:32Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.deployment.annotations.analyzer;

import java.util.HashMap;
import java.util.Map;

import org.objectweb.asm.AnnotationVisitor;
import org.objectweb.asm.MethodVisitor;
import org.objectweb.asm.commons.EmptyVisitor;

import org.objectweb.easybeans.deployment.annotations.JMethod;
import org.objectweb.easybeans.deployment.annotations.analyzer.method.JavaxAnnotationPostConstructVisitor;
import org.objectweb.easybeans.deployment.annotations.analyzer.method.JavaxAnnotationPreDestroyVisitor;
import org.objectweb.easybeans.deployment.annotations.analyzer.method.JavaxEjbAroundInvokeVisitor;
import org.objectweb.easybeans.deployment.annotations.analyzer.method.JavaxEjbExcludeClassInterceptorsVisitor;
import org.objectweb.easybeans.deployment.annotations.analyzer.method.JavaxEjbInitVisitor;
import org.objectweb.easybeans.deployment.annotations.analyzer.method.JavaxEjbPostActivateVisitor;
import org.objectweb.easybeans.deployment.annotations.analyzer.method.JavaxEjbPrePassivateVisitor;
import org.objectweb.easybeans.deployment.annotations.analyzer.method.JavaxEjbRemoveVisitor;
import org.objectweb.easybeans.deployment.annotations.analyzer.method.JavaxEjbTimeoutVisitor;
import org.objectweb.easybeans.deployment.annotations.metadata.ClassAnnotationMetadata;
import org.objectweb.easybeans.deployment.annotations.metadata.MethodAnnotationMetadata;

/**
 * This classes analyses a given method and build/fill meta data information.
 * @author Florent Benoit
 */
public class ScanMethodVisitor extends EmptyVisitor implements MethodVisitor {

    /**
     * Annotation visitor which do nothing.
     */
    private EmptyVisitor emptyVisitor = null;

    /**
     * Class generated by the visitor which correspond to meta data contained in the parsed method.
     */
    private MethodAnnotationMetadata methodAnnotationMetadata = null;


    /**
     * Map of annotation visitors used by this visitor.<br>
     * The key is the annotation descriptor (asm)
     */
    private Map<String, AnnotationVisitor> methodAnnotationVisitors = null;

    /**
     * Parent of method annotation meta data that are built by this visitor.
     */
    private ClassAnnotationMetadata classAnnotationMetadata = null;

    /**
     * Constructor.
     * @param jMethod Method object on which we set meta data.
     * @param classAnnotationMetadata the parent object on which add generated meta-data.
     */
    public ScanMethodVisitor(final JMethod jMethod, final ClassAnnotationMetadata classAnnotationMetadata) {

        // object build and to fill
        this.methodAnnotationMetadata = new MethodAnnotationMetadata(jMethod, classAnnotationMetadata);

        // parent
        this.classAnnotationMetadata = classAnnotationMetadata;

        // list of visitors to use
        methodAnnotationVisitors = new HashMap<String, AnnotationVisitor>();
        initVisitors();
    }

    /**
     * Build visitors used by this one.
     */
    private void initVisitors() {
        emptyVisitor = new EmptyVisitor();

        // add @Init
        methodAnnotationVisitors.put(JavaxEjbInitVisitor.TYPE, new JavaxEjbInitVisitor(methodAnnotationMetadata));

        // add @Remove
        methodAnnotationVisitors.put(JavaxEjbRemoveVisitor.TYPE, new JavaxEjbRemoveVisitor(methodAnnotationMetadata));

        // add @TransactionAttribute
        methodAnnotationVisitors.put(JavaxEjbTransactionAttributeVisitor.TYPE,
                new JavaxEjbTransactionAttributeVisitor<MethodAnnotationMetadata>(methodAnnotationMetadata));

        // add @PostConstruct
        methodAnnotationVisitors.put(JavaxAnnotationPostConstructVisitor.TYPE,
                new JavaxAnnotationPostConstructVisitor(methodAnnotationMetadata));
        // add @PreDestroy
        methodAnnotationVisitors.put(JavaxAnnotationPreDestroyVisitor.TYPE,
                new JavaxAnnotationPreDestroyVisitor(methodAnnotationMetadata));
        // add @PostActivate
        methodAnnotationVisitors.put(JavaxEjbPostActivateVisitor.TYPE, new JavaxEjbPostActivateVisitor(methodAnnotationMetadata));
        // add @PrePassivate
        methodAnnotationVisitors.put(JavaxEjbPrePassivateVisitor.TYPE, new JavaxEjbPrePassivateVisitor(methodAnnotationMetadata));

        // add @Timeout
        methodAnnotationVisitors.put(JavaxEjbTimeoutVisitor.TYPE, new JavaxEjbTimeoutVisitor(methodAnnotationMetadata));

        // add @AroundInvoke
        methodAnnotationVisitors.put(JavaxEjbAroundInvokeVisitor.TYPE, new JavaxEjbAroundInvokeVisitor(methodAnnotationMetadata));

        // add @EJB
        methodAnnotationVisitors.put(JavaxAnnotationEJBVisitor.TYPE,
                new JavaxAnnotationEJBVisitor<MethodAnnotationMetadata>(methodAnnotationMetadata));

        // add @Resource
        methodAnnotationVisitors.put(JavaxAnnotationResourceVisitor.TYPE,
                new JavaxAnnotationResourceVisitor<MethodAnnotationMetadata>(methodAnnotationMetadata));

        // add @Interceptors
        methodAnnotationVisitors.put(JavaxEjbInterceptorsVisitor.TYPE,
                new JavaxEjbInterceptorsVisitor<MethodAnnotationMetadata>(methodAnnotationMetadata));

        // add @ExcludeClassInterceptors
        methodAnnotationVisitors.put(JavaxEjbExcludeClassInterceptorsVisitor.TYPE,
                new JavaxEjbExcludeClassInterceptorsVisitor(methodAnnotationMetadata));

    }

    /**
     * Visits the default value of this annotation interface method.
     *
     * @return a non null visitor to the visit the actual default value of this
     *         annotation interface method. The 'name' parameters passed to the
     *         methods of this annotation visitor are ignored. Moreover, exacly
     *         one visit method must be called on this annotation visitor,
     *         followed by visitEnd.
     */
    @Override
    public AnnotationVisitor visitAnnotationDefault() {
        return null;
    }

    /**
     * Visits an annotation of this method.
     *
     * @param desc the class descriptor of the annotation class.
     * @param visible <tt>true</tt> if the annotation is visible at runtime.
     * @return a non null visitor to visit the annotation values.
     */
    @Override
    public AnnotationVisitor visitAnnotation(final String desc, final boolean visible) {
        AnnotationVisitor av = methodAnnotationVisitors.get(desc);
        if (av != null) {
            return av;
        }
        return emptyVisitor;
    }

    /**
     * Visits the end of the method. This method, which is the last one to be
     * called, is used to inform the visitor that all the annotations and
     * attributes of the method have been visited.
     */
    @Override
    public void visitEnd() {
        classAnnotationMetadata.addMethodAnnotationMetadata(methodAnnotationMetadata);
    }

}
