/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JavaxEjbMessageDrivenVisitor.java 9 2006-02-19 18:53:32Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.deployment.annotations.analyzer.classes;

import static org.objectweb.easybeans.deployment.annotations.ClassType.MDB;

import org.objectweb.asm.AnnotationVisitor;
import org.objectweb.asm.Type;
import org.objectweb.asm.commons.EmptyVisitor;

import org.objectweb.easybeans.deployment.annotations.analyzer.AbsAnnotationVisitor;
import org.objectweb.easybeans.deployment.annotations.analyzer.AnnotationType;
import org.objectweb.easybeans.deployment.annotations.impl.JActivationConfigProperty;
import org.objectweb.easybeans.deployment.annotations.impl.JMessageDriven;
import org.objectweb.easybeans.deployment.annotations.metadata.ClassAnnotationMetadata;
/**
 * This class manages the handling of &#64;{@link javax.ejb.MessageDriven} annotation.
 * @author Florent Benoit
 */
public class JavaxEjbMessageDrivenVisitor extends AbsAnnotationVisitor<ClassAnnotationMetadata> implements AnnotationType {

    /**
     * Type of annotation.
     */
    public static final String TYPE = "Ljavax/ejb/MessageDriven;";

    /**
     * Name attribute of the annotation.
     */
    private static final String NAME = "name";

    /**
     * messageListenerInterface attribute of the annotation.
     */
    private static final String MESSAGE_LISTENER_INTERFACE = "messageListenerInterface";

    /**
     * activationConfig attribute of the annotation.
     */
    private static final String ACTIVATION_CONFIG = "activationConfig";

    /**
     * Unqualified name of the bean.
     */
    private String unqualifiedName = null;

    /**
     * Message listener Interface.
     */
    private String messageListenerInterface = null;

    /**
     * Constructor.
     * @param classAnnotationMetadata linked to a class metadata
     */
    public JavaxEjbMessageDrivenVisitor(final ClassAnnotationMetadata classAnnotationMetadata) {
        super(classAnnotationMetadata);
    }

    /**
     * Visits a primitive value of the annotation.<br>
     * @param name the value name.
     * @param value the actual value, whose type must be {@link Byte},
     *        {@link Boolean}, {@link Character}, {@link Short},
     *        {@link Integer}, {@link Long}, {@link Float}, {@link Double},
     *        {@link String} or {@link Type}.
     */
    @Override
    public void visit(final String name, final Object value) {
        if (name.equals(NAME)) {
            this.unqualifiedName = (String) value;
        } else if (name.equals(MESSAGE_LISTENER_INTERFACE)) {
            this.messageListenerInterface = ((Type) value).getInternalName();
        }
    }

    /**
     * Visits an enumeration value of the annotation.
     * @param name the value name.
     * @param desc the class descriptor of the enumeration class.
     * @param value the actual enumeration value.
     */
    @Override
    public void visitEnum(final String name, final String desc, final String value) {

    }

    /**
     * Visits a nested annotation value of the annotation.
     * @param name the value name.
     * @param desc the class descriptor of the nested annotation class.
     * @return a non null visitor to visit the actual nested annotation value.
     *         <i>The nested annotation value must be fully visited before
     *         calling other methods on this annotation visitor</i>.
     */
    @Override
    public AnnotationVisitor visitAnnotation(final String name, final String desc) {
        return this;
    }

    /**
     * Visits an array value of the annotation.
     * @param name the value name.
     * @return a non null visitor to visit the actual array value elements. The
     *         'name' parameters passed to the methods of this visitor are
     *         ignored. <i>All the array values must be visited before calling
     *         other methods on this annotation visitor</i>.
     */
    @Override
    public AnnotationVisitor visitArray(final String name) {
        if (name.equals(ACTIVATION_CONFIG)) {
            return new ActivationConfigPropertyVisitor();
        }
        return this;
    }

    /**
     * Visits the end of the annotation. Creates the object and store it
     */
    @Override
    public void visitEnd() {

        // JMessageDriven object
        JMessageDriven jMessageDriven = getAnnotationMetadata().getJMessageDriven();
        if (jMessageDriven == null) {
            jMessageDriven = new JMessageDriven();
            getAnnotationMetadata().setJMessageDriven(jMessageDriven);
        }

        // message listener interface
        jMessageDriven.setMessageListenerInterface(messageListenerInterface);

        // unqualified name
        getAnnotationMetadata().setUnQualifiedBeanName(unqualifiedName);

        // Set type
        getAnnotationMetadata().setClassType(MDB);

    }

    /**
     * @return type of the annotation (its description)
     */
    public String getType() {
        return TYPE;
    }

    /**
     * Classes manages the parsing of activationConfig[] array of
     * &#64;{@link javax.ejb.MessageDriven} annotation.
     * @author Florent Benoit
     */
    class ActivationConfigPropertyVisitor extends EmptyVisitor {

        /**
         * Attribute for property name.
         */
        private static final String PROPERTY_NAME = "propertyName";

        /**
         * Attribute for property value.
         */
        private static final String PROPERTY_VALUE = "propertyValue";

        /**
         * Property name.
         */
        private String propertyName = null;

        /**
         * Property value.
         */
        private String propertyValue = null;


        /**
         * Visits a primitive value of the annotation.
         * @param name the value name.
         * @param value the actual value, whose type must be {@link Byte},
         *        {@link Boolean}, {@link Character}, {@link Short},
         *        {@link Integer}, {@link Long}, {@link Float},
         *        {@link Double}, {@link String} or {@link Type}.
         */
        @Override
        public void visit(final String name, final Object value) {
            if (name.equals(PROPERTY_NAME)) {
                propertyName = (String) value;
            } else if (name.equals(PROPERTY_VALUE)) {
                propertyValue = (String) value;
            }
        }

        /**
         * Visits the end of the annotation. Creates the object and store it
         */
        @Override
        public void visitEnd() {
            // Add an activation property
            JActivationConfigProperty jActivationConfigProperty = new JActivationConfigProperty(propertyName, propertyValue);
            JMessageDriven jMessageDriven = getAnnotationMetadata().getJMessageDriven();
            if (jMessageDriven == null) {
                jMessageDriven = new JMessageDriven();
                getAnnotationMetadata().setJMessageDriven(jMessageDriven);
            }
            jMessageDriven.addActivationConfigProperty(jActivationConfigProperty);

        }
    }

}
