/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: InheritanceMethodResolver.java 9 2006-02-19 18:53:32Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.deployment.annotations.helper.bean;

import org.objectweb.asm.Type;

import org.objectweb.easybeans.deployment.annotations.JMethod;
import org.objectweb.easybeans.deployment.annotations.exceptions.ResolverException;
import org.objectweb.easybeans.deployment.annotations.metadata.ClassAnnotationMetadata;
import org.objectweb.easybeans.deployment.annotations.metadata.EjbJarAnnotationMetadata;
import org.objectweb.easybeans.deployment.annotations.metadata.MethodAnnotationMetadata;

/**
 * This class adds method meta data to bean class from the super class.<br>
 * TODO: Try to analyze super class from a super classloader if not found in the
 * current ejb-jar
 * @see <a href="http://www.jcp.org/en/jsr/detail?id=220">EJB 3.0 Spec ?4.6.2</a><br><p>
 *      A super class can't be a bean class (stateless, stateful, etc) so the
 *      method metadata don't need to be cloned</p>
 * @author Florent Benoit
 */
public final class InheritanceMethodResolver {

    /**
     * java.lang.object internal name.
     */
    private static final String JAVA_LANG_OBJECT = Type.getInternalName(Object.class);

    /**
     * Helper class, no public constructor.
     */
    private InheritanceMethodResolver() {

    }

    /**
     * Found all method meta data of the super class and adds them to the class
     * being analyzed.
     * @param classAnnotationMetadata class to analyze
     * @throws ResolverException if the super class in not in the given ejb-jar
     */
    public static void resolve(final ClassAnnotationMetadata classAnnotationMetadata) throws ResolverException {
        addMethodMetadata(classAnnotationMetadata, classAnnotationMetadata);
    }

    /**
     * Adds method meta data on the first class by iterating on the second given
     * class.
     * @param beanclassAnnotationMetadata class where to add method metadata
     * @param visitingClassAnnotationMetadata takes method metadata from super
     *        class of the given class
     * @throws ResolverException if a super class metadata is not found from
     *         ejb-jar
     */
    private static void addMethodMetadata(final ClassAnnotationMetadata beanclassAnnotationMetadata,
            final ClassAnnotationMetadata visitingClassAnnotationMetadata) throws ResolverException {

        // Analyze super classes of the given class
        String superClass = visitingClassAnnotationMetadata.getSuperName();

        if (superClass != null) {

            // If super class is java.lang.Object, break the loop
            if (superClass.equals(JAVA_LANG_OBJECT)) {
                return;
            }

            // Get parent meta-data
            EjbJarAnnotationMetadata ejbJarAnnotationMetadata = beanclassAnnotationMetadata.getEjbJarAnnotationMetadata();

            // Get meta data of the super class
            ClassAnnotationMetadata superClassMetadata = ejbJarAnnotationMetadata.getClassAnnotationMetadata(superClass);

            if (superClassMetadata == null) {
                // TODO : I18n
                throw new ResolverException("The class " + beanclassAnnotationMetadata + " extends the class " + superClass
                        + "but this class seems to be outside of the ejb-jar");
            }

            // Takes method metadata of the super class and adds them to the
            // bean class
            // Note : the flag inherited is set to true
            for (MethodAnnotationMetadata methodAnnotationMetadata : superClassMetadata.getMethodAnnotationMetadataCollection()) {
                // check that the method has not be redefined
                JMethod method = methodAnnotationMetadata.getJMethod();
                if (beanclassAnnotationMetadata.getMethodAnnotationMetadata(method) == null) {
                    // Add method to bean class
                    beanclassAnnotationMetadata.addMethodAnnotationMetadata(methodAnnotationMetadata);

                    // method is inherited
                    methodAnnotationMetadata.setInherited(true);
                }
            }

            // Loop again
            addMethodMetadata(beanclassAnnotationMetadata, superClassMetadata);

        }
    }

}
