/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: TransactionResolver.java 14 2006-02-21 14:08:40Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.deployment.annotations.helper.bean;

import static javax.ejb.TransactionManagementType.BEAN;

import java.util.ArrayList;
import java.util.List;

import javax.ejb.TransactionAttributeType;
import javax.ejb.TransactionManagementType;

import org.objectweb.asm.Type;
import org.objectweb.easybeans.deployment.annotations.JClassInterceptor;
import org.objectweb.easybeans.deployment.annotations.JMethod;
import org.objectweb.easybeans.deployment.annotations.metadata.ClassAnnotationMetadata;
import org.objectweb.easybeans.deployment.annotations.metadata.MethodAnnotationMetadata;
import org.objectweb.easybeans.transaction.interceptors.BMTTransactionInterceptor;
import org.objectweb.easybeans.transaction.interceptors.CMTMandatoryTransactionInterceptor;
import org.objectweb.easybeans.transaction.interceptors.CMTNeverTransactionInterceptor;
import org.objectweb.easybeans.transaction.interceptors.CMTNotSupportedTransactionInterceptor;
import org.objectweb.easybeans.transaction.interceptors.CMTRequiredTransactionInterceptor;
import org.objectweb.easybeans.transaction.interceptors.CMTRequiresNewTransactionInterceptor;
import org.objectweb.easybeans.transaction.interceptors.CMTSupportsTransactionInterceptor;
import org.objectweb.easybeans.transaction.interceptors.ListenerSessionSynchronizationInterceptor;

/**
 * This class adds the interceptor for transaction on a given method.
 * @author Florent Benoit
 */
public final class TransactionResolver {

    /**
     * Signature of EasyBeans interceptors.
     */
    private static final JMethod EASYBEANS_INTERCEPTOR = new JMethod(0, "intercept",
            "(Lorg/objectweb/easybeans/api/EasyBeansInvocationContext;)Ljava/lang/Object;", null,
            new String[] {"java/lang/Exception"});

    /**
     * CMT Required transaction interceptor.
     */
    private static final String CMT_REQUIRED_INTERCEPTOR = Type
            .getInternalName(CMTRequiredTransactionInterceptor.class);

    /**
     * CMT Mandatory transaction interceptor.
     */
    private static final String CMT_MANDATORY_INTERCEPTOR = Type
            .getInternalName(CMTMandatoryTransactionInterceptor.class);

    /**
     * CMT Never transaction interceptor.
     */
    private static final String CMT_NEVER_INTERCEPTOR = Type.getInternalName(CMTNeverTransactionInterceptor.class);

    /**
     * CMT NotSupported transaction interceptor.
     */
    private static final String CMT_NOT_SUPPORTED_INTERCEPTOR = Type
            .getInternalName(CMTNotSupportedTransactionInterceptor.class);

    /**
     * CMT Supports transaction interceptor.
     */
    private static final String CMT_SUPPORTS_INTERCEPTOR = Type
            .getInternalName(CMTSupportsTransactionInterceptor.class);

    /**
     * CMT RequiresNew transaction interceptor.
     */
    private static final String CMT_REQUIRES_NEW_INTERCEPTOR = Type
            .getInternalName(CMTRequiresNewTransactionInterceptor.class);

    /**
     * BMT transaction interceptor.
     */
    private static final String BMT_INTERCEPTOR = Type.getInternalName(BMTTransactionInterceptor.class);

    /**
     * ListenerSessionSynchronizationInterceptor transaction interceptor.
     */
    private static final String LISTENER_SESSION_SYNCHRO_INTERCEPTOR = Type
            .getInternalName(ListenerSessionSynchronizationInterceptor.class);


    /**
     * javax.ejb.SessionSynchronization interface.
     */
    private static final String SESSION_SYNCHRONIZATION_INTERFACE = "javax/ejb/SessionSynchronization";

    /**
     * Helper class, no public constructor.
     */
    private TransactionResolver() {
    }

    /**
     * Adds the right transaction interceptor depending of the transactional
     * attribute set by the user.
     * @param bean the given bean on which set the transactional interceptor.
     */
    public static void resolve(final ClassAnnotationMetadata bean) {
        TransactionAttributeType beanTxType = bean.getTransactionAttributeType();
        TransactionManagementType beanTxManaged = bean.getTransactionManagementType();


        // Checks if Synchronization is needed for this stateful bean
        boolean addSynchro = false;
        if (bean.isStateful()) {
                String[] interfaces = bean.getInterfaces();
                if (interfaces != null) {
                        for (String itf : interfaces) {
                            if (SESSION_SYNCHRONIZATION_INTERFACE.equals(itf)) {
                                addSynchro = true;
                                break;
                            }
                        }
                }

        }

        for (MethodAnnotationMetadata method : bean.getMethodAnnotationMetadataCollection()) {

            List<JClassInterceptor> interceptors = method.getInterceptors();
            if (interceptors == null) {
                interceptors = new ArrayList<JClassInterceptor>();
            }

            // Bean managed or container managed ?
            if (beanTxManaged.equals(BEAN)) {
                // BMT
                interceptors.add(new JClassInterceptor(BMT_INTERCEPTOR, EASYBEANS_INTERCEPTOR));
            } else {
                // CMT
                TransactionAttributeType methodTx = method.getTransactionAttributeType();

                // Set method tx attribute to the class tx attribute if none was
                // set.
                if (methodTx == null) {
                    methodTx = beanTxType;
                }

                switch (methodTx) {
                    case MANDATORY:
                        interceptors.add(new JClassInterceptor(CMT_MANDATORY_INTERCEPTOR, EASYBEANS_INTERCEPTOR));
                        break;
                    case NEVER:
                        interceptors.add(new JClassInterceptor(CMT_NEVER_INTERCEPTOR, EASYBEANS_INTERCEPTOR));
                        break;
                    case NOT_SUPPORTED:
                        interceptors.add(new JClassInterceptor(CMT_NOT_SUPPORTED_INTERCEPTOR, EASYBEANS_INTERCEPTOR));
                        break;
                    case REQUIRED:
                        interceptors.add(new JClassInterceptor(CMT_REQUIRED_INTERCEPTOR, EASYBEANS_INTERCEPTOR));
                        break;
                    case REQUIRES_NEW:
                        interceptors.add(new JClassInterceptor(CMT_REQUIRES_NEW_INTERCEPTOR, EASYBEANS_INTERCEPTOR));
                        break;
                    case SUPPORTS:
                        interceptors.add(new JClassInterceptor(CMT_SUPPORTS_INTERCEPTOR, EASYBEANS_INTERCEPTOR));
                        break;
                    default:
                        throw new IllegalStateException("Invalid tx attribute on method '" + method.getMethodName()
                                + "', value = '" + methodTx + "'.");
                }

                // Add listener interceptor for stateul bean only if the bean implements SessionSynchronization interface
                if (addSynchro) {
                    interceptors.add(new JClassInterceptor(LISTENER_SESSION_SYNCHRO_INTERCEPTOR, EASYBEANS_INTERCEPTOR));
                }
                // End CMT
            }
            method.setInterceptors(interceptors);
        }
    }
}
