/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: InterceptorsValidator.java 9 2006-02-19 18:53:32Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.deployment.annotations.helper.bean.session.checks;

import java.util.Arrays;

import org.objectweb.easybeans.deployment.annotations.JMethod;
import org.objectweb.easybeans.deployment.annotations.exceptions.InterceptorsValidationException;
import org.objectweb.easybeans.deployment.annotations.impl.JInterceptors;
import org.objectweb.easybeans.deployment.annotations.metadata.ClassAnnotationMetadata;
import org.objectweb.easybeans.deployment.annotations.metadata.EjbJarAnnotationMetadata;
import org.objectweb.easybeans.deployment.annotations.metadata.MethodAnnotationMetadata;

/**
 * This class ensures that the interceptors have the correct signature.
 * @author Florent Benoit
 */
public final class InterceptorsValidator {

    /**
     * Signature for an AroundInvoke interceptor (InvocationContext).
     */
    private static final String AROUND_INVOKE_DESCRIPTOR_EJB = "(Ljavax/ejb/InvocationContext;)Ljava/lang/Object;";

    /**
     * Signature for a lifecycle interceptor inside the bean (void type).
     */
    private static final String LIFECYCLE_DESCRIPTOR_OUTSIDEBEAN = "(Ljavax/ejb/InvocationContext;)V";

    /**
     * Signature for a lifecycle interceptor inside the bean (void type).
     */
    private static final String LIFECYCLE_DESCRIPTOR_BEAN = "()V";

    /**
     * Exception required in AroundInvoke interceptor.
     */
    private static final String AROUND_INVOKE_EXCEPTION = "java/lang/Exception";

    /**
     * Helper class, no public constructor.
     */
    private InterceptorsValidator() {
    }

    /**
     * Validate a bean.
     * @param bean bean to validate.
     */
    public static void validate(final ClassAnnotationMetadata bean) {

        // Root metadata
        EjbJarAnnotationMetadata ejbMetaData = bean.getEjbJarAnnotationMetadata();

        // Interceptors in the bean
        for (MethodAnnotationMetadata method : bean.getMethodAnnotationMetadataCollection()) {

            // lifecycle
            if (method.isLifeCycleMethod()) {
                validateJMethod(method.getJMethod(), LIFECYCLE_DESCRIPTOR_BEAN, null, bean.getClassName());
            } else if (method.isAroundInvoke()) {
                validateJMethod(method.getJMethod(), AROUND_INVOKE_DESCRIPTOR_EJB, AROUND_INVOKE_EXCEPTION, bean
                        .getClassName());
            }

            // Interceptors defined on the bean's methods
            JInterceptors methodInterceptors = method.getAnnotationInterceptors();
            // Look in the interceptor class
            if (methodInterceptors != null) {
                for (String className : methodInterceptors.getClasses()) {
                    analyzeInterceptorClass(ejbMetaData, className);
                }
            }
        }

        // Now, check interceptors outside the bean
        JInterceptors methodInterceptors = bean.getAnnotationInterceptors();
        // Look in the interceptor class
        if (methodInterceptors != null) {
            for (String className : methodInterceptors.getClasses()) {
                analyzeInterceptorClass(ejbMetaData, className);
            }
        }
    }

    /**
     * Analyze an interceptor class and check the interceptors method.
     * @param ejbMetaData root metadata used to extract class metadata
     * @param className the name of the class being analyzed
     */
    private static void analyzeInterceptorClass(final EjbJarAnnotationMetadata ejbMetaData, final String className) {
        // get the metadata of the class
        ClassAnnotationMetadata interceptorMetaData = ejbMetaData.getClassAnnotationMetadata(className);
        if (interceptorMetaData == null) {
            throw new InterceptorsValidationException("Internal problem as no metadata was found for '" + className
                    + "'.");
        }

        for (MethodAnnotationMetadata method : interceptorMetaData.getMethodAnnotationMetadataCollection()) {

            // lifecycle
            if (method.isLifeCycleMethod()) {
                validateJMethod(method.getJMethod(), LIFECYCLE_DESCRIPTOR_OUTSIDEBEAN, null, className);
            } else if (method.isAroundInvoke()) {
                validateJMethod(method.getJMethod(), AROUND_INVOKE_DESCRIPTOR_EJB, AROUND_INVOKE_EXCEPTION, className);
            }

        }

    }

    /**
     * Validate the given method with the given signature/exceptions.
     * @param jMethod method to check.
     * @param desc signature to ensure.
     * @param awaitedException exception to ensure.
     * @param className the name of the class of the given method.
     */
    private static void validateJMethod(final JMethod jMethod, final String desc, final String awaitedException,
            final String className) {

        // validate signature
        if (!jMethod.getDescriptor().equals(desc)) {
            throw new InterceptorsValidationException("Method '" + jMethod + "' of the class '" + className
                    + "' is not compliant with the signature '" + desc + "'. Signature found = '"
                    + jMethod.getDescriptor() + "'.");
        }

        // validate exceptions
        String[] exceptions = jMethod.getExceptions();
        if (awaitedException == null) {
            // No exception, this is the expected behavior
            if (exceptions == null) {
                return;
            }

            // One exception is present, invalid !
            if (exceptions.length > 0) {
                throw new InterceptorsValidationException("Method '" + jMethod + "' of the class '" + className
                        + "' is not compliant with the signature '" + desc
                        + "' as no exception is required, found exception = '" + Arrays.asList(exceptions) + "'.");
            }
        }

        boolean found = false;

        if (exceptions != null) {
            for (String exception : exceptions) {
                if (exception.equals(awaitedException)) {
                    found = true;
                }
            }
        }
        if (!found) {
            throw new InterceptorsValidationException("Method '" + jMethod + "' of the class '" + className
                    + "' is not compliant with the signature '" + desc + "' as the required exception '"
                    + awaitedException + "' is missing.");
        }
    }

}
