/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: FieldAnnotationMetadata.java 9 2006-02-19 18:53:32Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.deployment.annotations.metadata;

import java.util.Arrays;

import org.objectweb.easybeans.deployment.annotations.JField;
import org.objectweb.easybeans.deployment.annotations.impl.JAnnotationEJB;
import org.objectweb.easybeans.deployment.annotations.impl.JAnnotationResource;
import org.objectweb.easybeans.deployment.annotations.metadata.interfaces.IAnnotationEJB;
import org.objectweb.easybeans.deployment.annotations.metadata.interfaces.IAnnotationResource;

/**
 * This class represents the annotation metadata of a field.
 * @author Florent Benoit
 */
public class FieldAnnotationMetadata implements IAnnotationEJB, IAnnotationResource {

    /**
     * Logger.
     */
    //private static JLog logger = JLogFactory.getLog(FieldAnnotationMetadata.class);

    /**
     * Method on which we got metadata.
     */
    private JField jField = null;

    /**
     * Parent metadata.
     */
    private ClassAnnotationMetadata classAnnotationMetadata = null;

    /**
     * Object representing &#64;{@link javax.annotation.EJB} annotation.
     */
    private JAnnotationEJB jAnnotationEJB = null;

    /**
     * Object representing &#64;{@link javax.annotation.Resource} annotation.
     */
    private JAnnotationResource jAnnotationResource = null;

    /**
     * This field is a field from a super class ?<br>
     */
    private boolean inherited = false;


    /**
     * This field is used as a PersistenceContext.
     */
    private boolean persistenceContext = false;

    /**
     * Constructor.
     * @param jField the field on which we will set/add metadata
     * @param classAnnotationMetadata the parent metadata.
     */
    public FieldAnnotationMetadata(final JField jField, final ClassAnnotationMetadata classAnnotationMetadata) {
        this.jField = jField;
        this.classAnnotationMetadata = classAnnotationMetadata;
    }

    /**
     * @return name of the field
     */
    public String getFieldName() {
        return this.jField.getName();
    }

    /**
     * @return JMethod object
     */
    public JField getJField() {
        return this.jField;
    }

    /**
     * @return string representation
     */
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        String titleIndent = " ";
        String indent = "   ";
        // classname
        sb.append(titleIndent);
        sb.append(this.getClass().getName().substring(this.getClass().getPackage().getName().length() + 1));
        sb.append("[\n");

        // Field
        sb.append(indent);
        sb.append("jField=");
        sb.append(jField);
        sb.append("\n");

        // jAnnotationEJB
        if (jAnnotationEJB != null) {
            sb.append(indent);
            sb.append("jAnnotationEJB=");
            sb.append(jAnnotationEJB);
            sb.append("\n");
        }

        // jAnnotationResource
        if (jAnnotationResource != null) {
            sb.append(indent);
            sb.append("jAnnotationResource=");
            sb.append(jAnnotationResource);
            sb.append("\n");
        }

        // inherited
        if (inherited) {
            sb.append(indent);
            sb.append("inherited=");
            sb.append(inherited);
            sb.append("\n");
        }

        // persistenceContext
        concatStringBuilder("persistenceContext", Boolean.valueOf(persistenceContext), sb);

        sb.append(titleIndent);
        sb.append("]\n");
        return sb.toString();
    }

    /**
     * Adds an entry to the given StringBuilder.
     * @param name the name of the entry.
     * @param object object to add.
     * @param sb the string builder object on which add the given element.
     * @param indent the indent to add at each line.
     */
    private static void concatStringBuilder(final String name, final Object object, final StringBuilder sb, final String indent) {
        if (object != null) {
            sb.append(indent);
            sb.append(name);
            sb.append("=");
            // For arrays, use asList
            if (object instanceof Object[]) {
                sb.append(Arrays.asList((Object[]) object));
            } else {
                sb.append(object);
            }
            sb.append("\n");
        }
    }

    /**
     * Adds an entry to the given StringBuilder.
     * @param name the name of the entry.
     * @param object object to add.
     * @param sb the string builder object on which add the given element.
     */
    private static void concatStringBuilder(final String name, final Object object, final StringBuilder sb) {
        concatStringBuilder(name, object, sb, "    ");
    }

    /**
     * @return true if this method is inherited from a super class
     */
    public boolean isInherited() {
        return inherited;
    }

    /**
     * Sets the inheritance of this method.
     * @param inherited true if method is from a super class
     */
    public void setInherited(final boolean inherited) {
        this.inherited = inherited;
    }

    /**
     * @return parent metadata (class)
     */
    public ClassAnnotationMetadata getClassAnnotationMetadata() {
        return classAnnotationMetadata;
    }


    /**
     * @return JAnnotationEJB object representing javax.annotation.EJB annotation.
     */
    public JAnnotationEJB getJAnnotationEJB() {
        return jAnnotationEJB;
    }

    /**
     * Set JAnnotationEJB object.
     * @param jAnnotationEJB object representing javax.annotation.EJB annotation.
     */
    public void setJAnnotationEJB(final JAnnotationEJB jAnnotationEJB) {
        this.jAnnotationEJB = jAnnotationEJB;
    }

    /**
     * @return JAnnotationResource object representing javax.annotation.Resource annotation.
     */
    public JAnnotationResource getJAnnotationResource() {
        return jAnnotationResource;
    }

    /**
     * Set JAnnotationResource object.
     * @param jAnnotationResource object representing javax.annotation.Resource annotation.
     */
    public void setJAnnotationResource(final JAnnotationResource jAnnotationResource) {
        this.jAnnotationResource = jAnnotationResource;
    }

    /**
     * @return true if this field is used as a persistence context.
     */
    public boolean isPersistenceContext() {
        return persistenceContext;
    }

    /**
     * Sets the persistence context info on this field.
     * @param persistenceContext true/false
     */
    public void setPersistenceContext(final boolean persistenceContext) {
        this.persistenceContext = persistenceContext;
    }

}
