/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: Enhancer.java 15 2006-02-21 18:06:07Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.enhancer;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.List;

import org.objectweb.asm.ClassReader;
import org.objectweb.asm.ClassWriter;

import org.objectweb.easybeans.deployment.annotations.metadata.ClassAnnotationMetadata;
import org.objectweb.easybeans.deployment.annotations.metadata.EjbJarAnnotationMetadata;
import org.objectweb.easybeans.enhancer.bean.BeanClassAdapter;
import org.objectweb.easybeans.enhancer.injection.InjectionClassAdapter;
import org.objectweb.easybeans.enhancer.interceptors.InterceptorClassAdapter;
import org.objectweb.easybeans.log.JLog;
import org.objectweb.easybeans.log.JLogFactory;

/**
 * This class is used for enhancing a set of classes (Beans like Stateless,
 * Stateful, MDB, etc).
 * @author Florent Benoit
 */
public class Enhancer {

    /**
     * Logger.
     */
    private static JLog logger = JLogFactory.getLog(Enhancer.class);

    /**
     * Metadata of the classes of a given ejb-jar.
     */
    private EjbJarAnnotationMetadata ejbJarAnnotationMetadata = null;

    /**
     * Classloader used to load/define classes.
     */
    private ClassLoader loader = null;

    /**
     * Creates an new enhancer.
     * @param loader classloader where to define enhanced classes.
     * @param ejbJarAnnotationMetadata object with references to the metadata.
     */
    public Enhancer(final ClassLoader loader, final EjbJarAnnotationMetadata ejbJarAnnotationMetadata) {
        this.loader = loader;
        this.ejbJarAnnotationMetadata = ejbJarAnnotationMetadata;
    }

    /**
     * Enhance all classes which match beans, etc.
     * @throws EnhancerException if enhancing fails
     */
    public void enhance() throws EnhancerException {

        // search all beans
        for (ClassAnnotationMetadata classAnnotationMetadata : ejbJarAnnotationMetadata
                .getClassAnnotationMetadataCollection()) {
            if (classAnnotationMetadata.isBean()) {

                // Create ClassReader/Writer
                ClassReader cr = getClassReader(classAnnotationMetadata);
                ClassWriter cw = new ClassWriter(true);
                BeanClassAdapter cv = new BeanClassAdapter(classAnnotationMetadata, cw);
                InjectionClassAdapter cv2 = new InjectionClassAdapter(classAnnotationMetadata, cv);
                InterceptorClassAdapter itcpClassAdapter = new InterceptorClassAdapter(classAnnotationMetadata, cv2);
                cr.accept(itcpClassAdapter, false);

                // define subclasses if interceptor enabled
                List<InterceptorClassAdapter.DefinedClass> lst = itcpClassAdapter.getDefinedClasses();
                if (lst != null) {
                    for (InterceptorClassAdapter.DefinedClass definedClass : lst) {
                        loadClass(definedClass.getClassName(), definedClass.getBytes());
                    }
                }

                loadClass(classAnnotationMetadata.getClassName().replace("/", "."), cw.toByteArray());

            } else if (classAnnotationMetadata.isInterceptor()) {
                logger.debug("ClassAdapter on interceptor : {0}", classAnnotationMetadata.getClassName());
                // Create ClassReader/Writer
                ClassReader cr = getClassReader(classAnnotationMetadata);
                ClassWriter cw = new ClassWriter(true);
                InterceptorClassAdapter cv = new InterceptorClassAdapter(classAnnotationMetadata, cw);
                cr.accept(cv, false);

                loadClass(classAnnotationMetadata.getClassName().replace("/", "."), cw.toByteArray());
            }

        }
    }


    /**
     * Gets a class reader for a given metadata.
     * @param classAnnotationMetadata given metadata
     * @return classreader associated to the given metadata
     * @throws EnhancerException if no classWriter can be returned
     */
    private ClassReader getClassReader(final ClassAnnotationMetadata classAnnotationMetadata) throws EnhancerException {
        String className = classAnnotationMetadata.getClassName() + ".class";
        InputStream is = loader.getResourceAsStream(className);
        if (is == null) {
            throw new EnhancerException("Cannot find input stream in classloader " + loader + " for class "
                    + className);
        }
        ClassReader cr = null;
        try {
            cr = new ClassReader(is);
        } catch (IOException e) {
            throw new EnhancerException("Cannot load input stream for class '" + className
                    + "' in classloader '" + loader);
        }
        return cr;
    }



    /**
     * Loads/defines a class in the current class loader.
     * @param className the name of the class
     * @param b the bytecode of the class to define
     */
    private static void loadClass(final String className, final byte[] b) {
        if (logger.isDebugEnabled()) {
            FileOutputStream fos = null;
            try {
                fos = new FileOutputStream(System.getProperty("java.io.tmpdir") + File.separator + className + ".class");
                logger.debug("Writing to " + fos);
                fos.write(b);
            } catch (IOException ioe) {
                throw new RuntimeException(ioe);
            }
        }

        // override classDefine (as it is protected) and define the class.
        try {
            ClassLoader loader = Thread.currentThread().getContextClassLoader();
            Class cls = Class.forName("java.lang.ClassLoader");
            java.lang.reflect.Method method = cls.getDeclaredMethod("defineClass", new Class[] {String.class,
                    byte[].class, int.class, int.class});

            // protected method invocaton
            method.setAccessible(true);
            try {
                Object[] args = new Object[] {className, b, new Integer(0), new Integer(b.length)};
                method.invoke(loader, args);
            } finally {
                method.setAccessible(false);
            }
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

}
