/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: BeanClassAdapter.java 9 2006-02-19 18:53:32Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.enhancer.bean;

import org.objectweb.asm.ClassAdapter;
import org.objectweb.asm.ClassVisitor;
import org.objectweb.asm.FieldVisitor;
import org.objectweb.asm.MethodVisitor;
import org.objectweb.asm.Opcodes;
import org.objectweb.asm.Type;

import org.objectweb.easybeans.api.bean.EasyBeansBean;
import org.objectweb.easybeans.api.bean.EasyBeansMDB;
import org.objectweb.easybeans.api.bean.EasyBeansSFSB;
import org.objectweb.easybeans.api.bean.EasyBeansSLSB;
import org.objectweb.easybeans.deployment.annotations.metadata.ClassAnnotationMetadata;

/**
 * This class adds a bean interface to the parsed object.<br>
 * It also adds getEasyBeansFactory method defined in the EasyBeansBean interface.<br>
 * Stateless bean will have EasyBeansStatelessSessionBean interface, etc.
 * @author Florent Benoit
 */
public class BeanClassAdapter extends ClassAdapter implements Opcodes {

    /**
     * Metadata available by this adapter for a class.
     */
    private ClassAnnotationMetadata classAnnotationMetadata = null;

    /**
     * Constructor.
     * @param classAnnotationMetadata object containing all attributes of the
     *        class
     * @param cv the class visitor to which this adapter must delegate calls.
     */
    public BeanClassAdapter(final ClassAnnotationMetadata classAnnotationMetadata, final ClassVisitor cv) {
        super(cv);
        this.classAnnotationMetadata = classAnnotationMetadata;
    }

    /**
     * Visits the header of the class.
     * @param version the class version.
     * @param access the class's access flags (see
     *        {@link org.objectweb.asm.Opcodes}). This parameter also indicates
     *        if the class is deprecated.
     * @param name the internal name of the class (see
     *        {@link org.objectweb.asm.Type#getInternalName() getInternalName}).
     * @param signature the signature of this class. May be <tt>null</tt> if
     *        the class is not a generic one, and does not extend or implement
     *        generic classes or interfaces.
     * @param superName the internal of name of the super class (see
     *        {@link org.objectweb.asm.Type#getInternalName() getInternalName}).
     *        For interfaces, the super class is {@link Object}. May be
     *        <tt>null</tt>, but only for the {@link Object} class.
     * @param interfaces the internal names of the class's interfaces (see
     *        {@link org.objectweb.asm.Type#getInternalName() getInternalName}).
     *        May be <tt>null</tt>.
     */
    @Override
    public void visit(final int version, final int access, final String name, final String signature,
            final String superName, final String[] interfaces) {

        String[] newInterfaces = null;

        // Add new interface with bean
        if (classAnnotationMetadata.isBean()) {
            // copy old interfaces in the new array
            newInterfaces = new String[interfaces.length + 1];
            System.arraycopy(interfaces, 0, newInterfaces, 0, interfaces.length);

            int indexElement = newInterfaces.length - 1;

            // Add the right interface (SLSB, SFSB, MDB)
            if (classAnnotationMetadata.isStateless()) {
                newInterfaces[indexElement] = Type.getInternalName(EasyBeansSLSB.class);
            } else if (classAnnotationMetadata.isStateful()) {
                newInterfaces[indexElement] = Type.getInternalName(EasyBeansSFSB.class);
            } else if (classAnnotationMetadata.isMdb()) {
                newInterfaces[indexElement] = Type.getInternalName(EasyBeansMDB.class);
            } else {
                newInterfaces[indexElement] = Type.getInternalName(EasyBeansBean.class);
            }
        } else {
            newInterfaces = interfaces;
        }

        super.visit(version, access, name, signature, superName, newInterfaces);

    }

    /**
     * Visits the end of the class. This method, which is the last one to be
     * called, is used to inform the visitor that all the fields and methods of
     * the class have been visited.
     */
    @Override
    public void visitEnd() {
        super.visitEnd();

        addFactory();


    }


    /**
     * Adds a factory attribute and its getter/setter.
     */
    private void addFactory() {
        // Add the easyBeansFactory attribute
        // private Factory easyBeansFactory = null;
        FieldVisitor fv = cv.visitField(ACC_PRIVATE, "easyBeansFactory", "Lorg/objectweb/easybeans/api/Factory;", null, null);
        fv.visitEnd();

        // Add its getter :
        // public Factory getEasyBeansFactory() {
        //    return this.easyBeansFactory;
        // }
        MethodVisitor mv = cv.visitMethod(ACC_PUBLIC, "getEasyBeansFactory", "()Lorg/objectweb/easybeans/api/Factory;", null, null);
        mv.visitCode();
        mv.visitVarInsn(ALOAD, 0);
        mv.visitFieldInsn(GETFIELD, classAnnotationMetadata.getClassName(),
                "easyBeansFactory", "Lorg/objectweb/easybeans/api/Factory;");
        mv.visitInsn(ARETURN);
        mv.visitMaxs(0, 0);
        mv.visitEnd();

        // Add the setter
        // public void setEasyBeansFactory(final Factory easyBeansFactory) {
        //    this.easyBeansFactory = easyBeansFactory;
        // }
        mv = cv.visitMethod(ACC_PUBLIC, "setEasyBeansFactory", "(Lorg/objectweb/easybeans/api/Factory;)V", null, null);
        mv.visitCode();
        mv.visitVarInsn(ALOAD, 0);
        mv.visitVarInsn(ALOAD, 1);
        mv.visitFieldInsn(PUTFIELD, classAnnotationMetadata.getClassName(),
                "easyBeansFactory", "Lorg/objectweb/easybeans/api/Factory;");
        mv.visitInsn(RETURN);
        mv.visitMaxs(0, 0);
        mv.visitEnd();
    }
}
