/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: InjectionClassAdapter.java 9 2006-02-19 18:53:32Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.enhancer.injection;

import org.objectweb.asm.ClassAdapter;
import org.objectweb.asm.ClassVisitor;
import org.objectweb.asm.MethodVisitor;
import org.objectweb.asm.Opcodes;

import org.objectweb.easybeans.deployment.annotations.metadata.ClassAnnotationMetadata;
import org.objectweb.easybeans.deployment.annotations.metadata.FieldAnnotationMetadata;
import org.objectweb.easybeans.log.JLog;
import org.objectweb.easybeans.log.JLogFactory;

/**
 * This class adds methods which will inject resources in the bean class.
 * @author Florent Benoit
 */
public class InjectionClassAdapter extends ClassAdapter implements Opcodes {

    /**
     * Logger.
     */
    private JLog logger = JLogFactory.getLog(InjectionClassAdapter.class);

    /**
     * Metadata available by this adapter for a class.
     */
    private ClassAnnotationMetadata classAnnotationMetadata;

    /**
     * Constructor.
     * @param classAnnotationMetadata object containing all attributes of the
     *        class
     * @param cv the class visitor to which this adapter must delegate calls.
     */
    public InjectionClassAdapter(final ClassAnnotationMetadata classAnnotationMetadata, final ClassVisitor cv) {
        super(cv);
        this.classAnnotationMetadata = classAnnotationMetadata;
    }

    /**
     * Visits the end of the class. This method, which is the last one to be
     * called, is used to inform the visitor that all the fields and methods of
     * the class have been visited.
     */
    @Override
    public void visitEnd() {
        super.visitEnd();

        // now, adds the injected method
        addInjectedMethod();
    }

    /**
     * Generates the injectedByEasyBeans() method on the current class.
     */
    private void addInjectedMethod() {
        MethodVisitor mv = cv.visitMethod(ACC_PUBLIC, "injectedByEasyBeans", "()V", null,
                new String[] {"org/objectweb/easybeans/api/bean/injection/EasyBeansInjectionException"});
        mv.visitCode();

        generateBodyInjectedMethod(mv);

        mv.visitInsn(RETURN);
        mv.visitMaxs(0, 0);
        mv.visitEnd();

    }

    /**
     * Generates the body of the injectedByEasyBeans() method if any.<br>
     * Else, do nothing.
     * @param mv the method visitor object used to add some code.
     */
    private void generateBodyInjectedMethod(final MethodVisitor mv) {
        for (FieldAnnotationMetadata fieldMetaData : classAnnotationMetadata.getFieldAnnotationMetadataCollection()) {
            if (fieldMetaData.isPersistenceContext()) {
                logger.debug("Add injection for PersistenceContext on attribute {0} of class {1}",
                        fieldMetaData.getFieldName(), classAnnotationMetadata.getClassName());
                // add this.em = EntityManagerHelper.getEntityManager(this);
                mv.visitVarInsn(ALOAD, 0);
                mv.visitVarInsn(ALOAD, 0);
                mv.visitMethodInsn(INVOKESTATIC, "org/objectweb/easybeans/injection/EntityManagerHelper",
                        "getEntityManager", "(Lorg/objectweb/easybeans/api/bean/EasyBeansBean;)Ljavax/persistence/EntityManager;");
                mv.visitFieldInsn(PUTFIELD, classAnnotationMetadata.getClassName(),
                        fieldMetaData.getFieldName(), "Ljavax/persistence/EntityManager;");
            }
        }
    }

}
