/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JPersistenceUnitInfoLoader.java 9 2006-02-19 18:53:32Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.persistence.xml;

import static javax.persistence.spi.PersistenceUnitTransactionType.JTA;
import static javax.persistence.spi.PersistenceUnitTransactionType.RESOURCE_LOCAL;

import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;


import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import org.objectweb.easybeans.log.JLog;
import org.objectweb.easybeans.log.JLogFactory;

/**
 * Class used to fill PersistenceUnitInfo implementation class by loading an
 * XML.
 * @author Florent Benoit
 */
public final class JPersistenceUnitInfoLoader {

    /**
     * Logger.
     */
    private static JLog logger = JLogFactory.getLog(JPersistenceUnitInfoLoader.class);

    /**
     * Validating with schema ?
     */
    private static boolean validating = true;

    /**
     * Utility class, no public constructor.
     */
    private JPersistenceUnitInfoLoader() {

    }

    /**
     * Load the persistence.xml file.
     * @param url the URL of the the Reader of the XML file.
     * @throws JPersistenceUnitInfoException if parsing of XML file fails.
     * @return an application object.
     */
    public static JPersistenceUnitInfo[] loadPersistenceUnitInfoImpl(final URL url)
            throws JPersistenceUnitInfoException {


        logger.debug("Analyzing url {0}", url);

        // List of PersistenceUnitInfo objects
        List<JPersistenceUnitInfo> jPersistenceUnitInfos = new ArrayList<JPersistenceUnitInfo>();

        // Get document
        Document document = getDocument(url);

        // Root element = <persistence>
        Element persistenceRootElement = document.getDocumentElement();

        // Get children of this element (persistenceUnit elements)
        NodeList persistenceUnitInfoList = XMLUtils.getPersistenceUnitElement(persistenceRootElement);

        // Loop on this list
        for (int i = 0; i < persistenceUnitInfoList.getLength(); i++) {
            Element pUnitElement = (Element) persistenceUnitInfoList.item(i);

            // Build instance that is created.
            JPersistenceUnitInfo persistenceUnitInfo = new JPersistenceUnitInfo();

            // set the URL of the persistence file.
            persistenceUnitInfo.setPersistenceXmlFileUrl(url);

            // Provider
            String className = XMLUtils.getStringValueElement(pUnitElement, "provider");
            persistenceUnitInfo.setPersistenceProviderClassName(className);

            // Jta-data-source
            String jtaDataSourceName = XMLUtils.getStringValueElement(pUnitElement, "jta-data-source");
            persistenceUnitInfo.setJtaDataSourceName(jtaDataSourceName);

            // Non-jta-data-source
            String nonJtaDataSourceName = XMLUtils.getStringValueElement(pUnitElement, "non-jta-data-source");
            persistenceUnitInfo.setNonJtaDataSourceName(nonJtaDataSourceName);

            // mapping-file
            List<String> mappingFiles = XMLUtils.getStringListValueElement(pUnitElement, "mapping-file");
            for (String mappingFileName : mappingFiles) {
                persistenceUnitInfo.addMappingFileName(mappingFileName);
            }

            //TODO: jarFile
            // jar-file
            //List<String> jarFiles = XMLUtils.getStringListValueElement(pUnitElement, "jar-file");
            //for (String jarFileName : jarFiles) {
                //persistenceUnitInfo.addJarFile(mappingFileName);
            //}


            // class (managed class)
            List<String> classes = XMLUtils.getStringListValueElement(pUnitElement, "class");
            for (String managedClassName : classes) {
                persistenceUnitInfo.addClass(managedClassName);
            }

            // exclude-unlisted-classes
            String excluded = XMLUtils.getStringValueElement(pUnitElement, "exclude-unlisted-classes");
            persistenceUnitInfo.setExcludeUnlistedClasses("true".equals(excluded));

            // properties
            Properties props = XMLUtils.getPropertiesValueElement(pUnitElement, "properties");
            persistenceUnitInfo.setProperties(props);

            // Name attribute
            String name = XMLUtils.getAttributeValue(pUnitElement, "name");
            persistenceUnitInfo.setPersistenceUnitName(name);

            // transaction-type attribute
            String transactionType = XMLUtils.getAttributeValue(pUnitElement, "transaction-type");
            if ("JTA".equals(transactionType)) {
                persistenceUnitInfo.setTransactionType(JTA);
            } else if ("RESOURCE_LOCAL".equals(transactionType)) {
                persistenceUnitInfo.setTransactionType(RESOURCE_LOCAL);
            }

            jPersistenceUnitInfos.add(persistenceUnitInfo);

        }
        return jPersistenceUnitInfos.toArray(new JPersistenceUnitInfo[jPersistenceUnitInfos.size()]);
    }

    /**
     * Builds a new Document for a given xml file.
     * @param url the URL of the the XML file.
     * @throws JPersistenceUnitInfoException if creating of builder fails.
     * @return an application object.
     */
    private static Document getDocument(final URL url) throws JPersistenceUnitInfoException {
        // build factory
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();

        // Schema is used for persistence.xsd
        factory.setNamespaceAware(true);
        factory.setValidating(validating);

        // ignore white space can only be set if parser is validating
        if (validating) {
            factory.setIgnoringElementContentWhitespace(validating);
            factory.setAttribute("http://apache.org/xml/features/validation/schema", new Boolean(validating));
        }
        // Build a document builder
        DocumentBuilder builder = null;
        try {
            builder = factory.newDocumentBuilder();
        } catch (ParserConfigurationException e) {
            throw new JPersistenceUnitInfoException("Cannot build a document builder", e);
        }

        // Add schema
        if (factory.isValidating()) {
            builder.setEntityResolver(new PersistenceEntityResolver());
        }

        // Parse document
        URLConnection urlConnection = null;
        try {
            urlConnection = url.openConnection();
        } catch (IOException e) {
            throw new JPersistenceUnitInfoException("Cannot open a connection on URL '" + url + "'", e);
        }
        urlConnection.setDefaultUseCaches(false);
        Reader reader = null;
        try {
            reader = new InputStreamReader(urlConnection.getInputStream());
        } catch (IOException e) {
            throw new JPersistenceUnitInfoException("Cannot build an input stream reader on URL '" + url + "'", e);
        }

        InputSource inputSource = new InputSource(reader);
        Document document = null;
        try {
            document = builder.parse(inputSource);
        } catch (SAXException e) {
            throw new JPersistenceUnitInfoException("Cannot parse the XML file '" + url + "'.", e);
        } catch (IOException e) {
            throw new JPersistenceUnitInfoException("Cannot parse the XML file '" + url + "'.", e);
        } finally {
            // close InputStream when parsing is finished
            try {
                reader.close();
            } catch (IOException e) {
                throw new JPersistenceUnitInfoException("Cannot close the inputsource of the XML file'" + url + "'.", e);
            }
        }

        return document;
    }

    /**
     * Class with some useful methods on XML document.
     */
    public static final class XMLUtils {

        /**
         * Persistence namespace.
         */
        private static final String PERSISTENCE_NS = "http://java.sun.com/xml/ns/persistence";

        /**
         * &lt;persistence-unit&gt; element.
         */
        private static final String PERSISTENCE_UNIT = "persistence-unit";

        /**
         * Utility class, no constructor.
         */
        private XMLUtils() {

        }

        /**
         * Returns all the <code>persistence-unit</code> XML elements.
         * @param root <code>root</code> Element.
         * @return the matching <code>persistence-unit</code> XML Element.
         */
        public static NodeList getPersistenceUnitElement(final Element root) {
            // List of persistence-units
            return root.getElementsByTagNameNS(PERSISTENCE_NS, PERSISTENCE_UNIT);
            // return persistenceUnits;
        }


        /**
         * Returns the value of the attribute of the given element.
         * @param base the element from where to search.
         * @param name of the attribute to get.
         * @return the value of this element.
         */
        public static String getAttributeValue(final Element base, final String name) {

            // get attribute of this element...
            NamedNodeMap mapAttributes = base.getAttributes();
            Node node = mapAttributes.getNamedItem(name);
            if (node != null) {
                return node.getNodeValue();
            }
            return null;
        }


        /**
         * Returns the value of the given node.
         * @param base the element from where to search.
         * @param name of the element to get.
         * @return the value of this element.
         */
        public static String getStringValueElement(final Element base, final String name) {
            String value = null;

            // Get element
            NodeList list = base.getElementsByTagNameNS(PERSISTENCE_NS, name);
            if (list.getLength() == 1) {
                Element element = (Element) list.item(0);
                Node node = element.getFirstChild();
                if (node != null) {
                    value = node.getNodeValue();
                }
            } else if (list.getLength() > 1) {
                throw new IllegalStateException("Element '" + name + "' on '" + base
                        + "' should be unique but there are '" + list.getLength() + "' elements");
            }

            return value;
        }

        /**
         * Returns a Properties object matching the given node.
         * @param base the element from where to search.
         * @param name of the element to get.
         * @return the value of this element.
         */
        public static Properties getPropertiesValueElement(final Element base, final String name) {
            Properties returnedProperties = new Properties();

            // Get element
            NodeList list = base.getElementsByTagNameNS(PERSISTENCE_NS, name);
            if (list.getLength() == 1) {
                Element element = (Element) list.item(0);

                // Get property element
                NodeList properties = element.getElementsByTagNameNS(PERSISTENCE_NS, "property");

                // If properties is present, analyze them and add them
                if (properties.getLength() > 0) {
                    for (int i = 0; i < properties.getLength(); i++) {
                        Element elemProperty = (Element) properties.item(i);
                        String pName = getAttributeValue(elemProperty, "name");
                        String pValue = getAttributeValue(elemProperty, "value");
                        if (pName != null && pValue != null) {
                            returnedProperties.setProperty(pName, pValue);
                        }

                    }
                }
            } else if (list.getLength() > 1) {
                throw new IllegalStateException("Element '" + name + "' on '" + base
                        + "' should be unique but there are '" + list.getLength() + "' elements");
            }

            return returnedProperties;
        }

        /**
         * Returns a list of value for the given node.
         * @param base the element from where to search.
         * @param name of the element to get.
         * @return the list of value of this element.
         */
        public static List<String> getStringListValueElement(final Element base, final String name) {
            List<String> returnedlist = new ArrayList<String>();

            // Get element
            NodeList list = base.getElementsByTagNameNS(PERSISTENCE_NS, name);
            int length = list.getLength();

            // Get all values of all elements
            if (length > 0) {
                for (int i = 0; i < length; i++) {
                    Element element = (Element) list.item(i);
                    Node node = element.getFirstChild();
                    if (node != null) {
                        returnedlist.add(node.getNodeValue());
                    }
                }
            }
            return returnedlist;
        }


    }

    /**
     * Entity resolver allowing to find schema within the classloader.
     */
    public static class PersistenceEntityResolver implements EntityResolver {

        /**
         * Persistence Schema.
         */
        public static final String PERSISTENCE_SCHEMA_10 = "persistence_1_0.xsd";

        /**
         * Local url of the schema.
         */
        private URL url = null;

        /**
         * Constructor. Finds the XSD with classloader.
         */
        public PersistenceEntityResolver() {
            url = PersistenceEntityResolver.class.getResource("/"
                    + PersistenceEntityResolver.class.getPackage().getName().replace(".", "/") + "/" + PERSISTENCE_SCHEMA_10);
            if (url == null) {
                throw new IllegalStateException("'" + PERSISTENCE_SCHEMA_10
                        + "' was not found in the current classloader !");
            }
        }

        /**
         * The Parser will call this method before opening any external entity
         * except the top-level document entity.
         * @param publicId The public identifier of the external entity being
         *        referenced, or null if none was supplied.
         * @param systemId The system identifier of the external entity being
         *        referenced.
         * @return An InputSource object describing the new input source, or
         *         null to request that the parser open a regular URI connection
         *         to the system identifier.
         * @throws SAXException Any SAX exception, possibly wrapping another
         *         exception.
         * @throws IOException A Java-specific IO exception, possibly the result
         *         of creating a new InputStream or Reader for the InputSource.
         */
        public InputSource resolveEntity(final String publicId, final String systemId) throws IOException, SAXException {

            String localPath = null;
            if (systemId != null) {
                // Can be a schema
                if (systemId.toLowerCase().endsWith(".xsd")) {
                    // Retrieve basename
                    String baseName = systemId.substring(systemId.lastIndexOf('/') + 1);

                    if (baseName.equals(PERSISTENCE_SCHEMA_10)) {
                        localPath = url.toString();
                    }
                }
            }

            // schema not found
            if (localPath == null) {
                throw new SAXException("No XSD found for '" + systemId + "'.");
            }

            // Return the local path source
            return (new InputSource(localPath));
        }

    }

}
