/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: Embedded.java 29 2006-02-22 16:58:22Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.easybeans.server;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.net.MalformedURLException;
import java.rmi.RemoteException;
import java.util.HashMap;
import java.util.Map;

import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.objectweb.carol.jndi.ns.NameServiceException;
import org.objectweb.carol.jndi.ns.NameServiceManager;
import org.objectweb.carol.util.configuration.ConfigurationException;
import org.objectweb.carol.util.configuration.ConfigurationRepository;

import org.objectweb.easybeans.api.EZBContainer;
import org.objectweb.easybeans.container.JContainer3;
import org.objectweb.easybeans.deployer.Deployer;
import org.objectweb.easybeans.ds.DummyDataSource;
import org.objectweb.easybeans.jmx.JMXRemoteException;
import org.objectweb.easybeans.jmx.JMXRemoteHelper;
import org.objectweb.easybeans.jmx.MBeanServerException;
import org.objectweb.easybeans.jmx.MBeanServerHelper;
import org.objectweb.easybeans.jmx.MBeansException;
import org.objectweb.easybeans.jmx.MBeansHelper;
import org.objectweb.easybeans.log.JLog;
import org.objectweb.easybeans.log.JLogFactory;
import org.objectweb.easybeans.rpc.rmi.server.RMIServerRPC;
import org.objectweb.easybeans.rpc.rmi.server.RMIServerRPCImpl;
import org.objectweb.easybeans.transaction.JTransactionManager;

/**
 * Allows to create an embedded EJB3 server.
 * @author Florent Benoit
 */
public class Embedded {

    /**
     * Default sleep value (for server loop).
     */
    private static final int SLEEP_VALUE = 10000;

    /**
     * Logger.
     */
    private static JLog logger = JLogFactory.getLog(Embedded.class);

    /**
     * Configuration of this server.
     */
    private ServerConfig config = null;

    /**
     * Map of managed ejb3 containers.
     */
    private Map<String, EZBContainer> containers = null;

    /**
     * Creates a new Embedded server.<br>
     * It will take default values of configuration.
     */
    public Embedded() {
        this(new ServerConfig());
    }

    /**
     * Creates a new embedded server with a given configuration.
     * @param config the server configuration.
     */
    public Embedded(final ServerConfig config) {
        this.config = config;
        this.containers = new HashMap<String, EZBContainer>();
        configure();

    }

    /**
     * Starts the EJB3 server.
     * @throws EmbeddedException if there is a failure while starting the
     *         server.
     */
    public void start() throws EmbeddedException {
        long tStart = System.currentTimeMillis();
        //System.setProperty("org.apache.commons.logging.Log", "org.apache.commons.logging.impl.SimpleLog");
        //System.setProperty("org.apache.commons.logging.simplelog.defaultlog", "trace");

        // start the registry ?
        if (config.startRegistry()) {
            // Set carol mode in server mode (will use fixed port if set)
            System.setProperty("carol.server.mode", "true");
            try {
                // hack
                Writer fw;
                File fConf = new File(System.getProperty("java.io.tmpdir") + File.separator + "ejb3-carol.properties");
                try {
                    fw = new FileWriter(fConf);
                    fw.write("carol.protocols=jrmp\n");
                    fw.write("carol.jrmp.url=rmi://localhost:1099\n");
                    fw.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }

                ConfigurationRepository.init(fConf.toURL());
            } catch (ConfigurationException e) {
                throw new EmbeddedException("Cannot initialize registry", e);
            } catch (MalformedURLException e) {
                throw new EmbeddedException("Cannot initialize registry", e);
            }

            if (config.startTM()) {
                try {
                    ConfigurationRepository.addInterceptors("jrmp", "org.objectweb.jotm.jta.rmi.JTAInterceptorInitializer");
                } catch (ConfigurationException e) {
                    throw new EmbeddedException("Cannot add JOTM interceptors", e);
                }
            }

            try {
                NameServiceManager.startNS();
            } catch (NameServiceException e) {
                throw new EmbeddedException("Cannot start registry", e);
            }

        }

        // set the deployer used for this component.
        Deployer.setEmbedded(this);

        if (config.isUsingMBeans()) {
            // Start MBeanServer (if any)
            try {
                MBeanServerHelper.startMBeanServer();
            } catch (MBeanServerException e) {
                throw new EmbeddedException("Cannot start MBean server", e);
            }

            try {
                JMXRemoteHelper.startConnector();
            } catch (JMXRemoteException e) {
                throw new EmbeddedException("Cannot start JMX Remote connector", e);
            }

            // if MBeans
            try {
                MBeansHelper.initMBeans();
            } catch (MBeansException e) {
                throw new EmbeddedException("Cannot init MBeans", e);
            }
        }


        // Bind the TM factory
        if (config.startTM()) {
            JTransactionManager.getTransactionManager();
        }


        // Bind a dummy datasource
        if (config.bindDummyDataSourceHSQL()) {
            try {
                new InitialContext().rebind("jdbc_1", new DummyDataSource());
            } catch (NamingException e) {
                throw new EmbeddedException("Cannot start the dummy In-memory Datasource", e);
            }
        }


        // Bind the RPC object
        RMIServerRPC invoker = null;
        try {
            invoker = new RMIServerRPCImpl(this);
        } catch (RemoteException e) {
            throw new EmbeddedException("Cannot build RPC invoker", e);
        }
        try {
            new InitialContext().rebind(RMIServerRPC.RPC_JNDI_NAME, invoker);
        } catch (NamingException e) {
            throw new EmbeddedException("Cannot bind the RPC invoker", e);
        }


        // add a thread which monitor containers file
        ContainersMonitor monitor = new ContainersMonitor(this);
        monitor.init();

        logger.info("Embedded.start.startup", Long.valueOf(System.currentTimeMillis() - tStart));
        logger.info("Embedded.start.created", Integer.valueOf(containers.size()));

        logger.info("Embedded.start.waiting");
        monitor.start();

        if (config.shouldWait()) {
            while (true) {
                try {
                    Thread.sleep(SLEEP_VALUE);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    /**
     * Stops the EJB3 server.
     */
    public void stop() {

    }

    /**
     * Gets a container managed by this server.
     * @param id the container id.
     * @return the container if it is found, else null.
     */
    public EZBContainer getContainer(final String id) {
        return containers.get(id);
    }

    /**
     * Configure the server by using the given configuration.
     */
    private void configure() {
        File ejb3Directory = config.getEjb3Directory();
        // need to define a repository
        if (ejb3Directory == null) {
            ejb3Directory = new File("ejb3s");
            logger.warn("No directory was configured, take the default value of '" + ejb3Directory.getAbsolutePath() + "'.");
            if (!ejb3Directory.exists()) {
                if (ejb3Directory.mkdir()) {
                    logger.warn("Directory '" + ejb3Directory.getAbsolutePath() + "' created.");
                } else {
                    throw new IllegalStateException("Cannot make directory '" + ejb3Directory.getAbsolutePath() + "'.");
                }
            }
            // update the value
            config.setEjb3Path(ejb3Directory.getAbsolutePath());
        }
    }


    /**
     * Creates and adds an ejb3 container to the managed container.
     * @param archive jar file or exploded archive.
     * @return the created container.
     */
    public EZBContainer createContainer(final File archive) {
        EZBContainer container = new JContainer3(archive);
        String id = container.getId();
        containers.put(id, container);
        return container;
    }

    /**
     * Main method called by default.
     * @param args the arguments for the main method
     * @throws Exception if failures
     */
    public static void main(final String[] args) throws Exception {

        // Configure server
        ServerConfig config = new ServerConfig();
        Embedded embedded = new Embedded(config);

        // Start new embedded server
        embedded.start();
    }


    /**
     * @return the configuration of this server.
     */
    protected ServerConfig getConfig() {
        return config;
    }

    /**
     * @return the containers managed by this server.
     */
    protected Map<String, EZBContainer> getContainers() {
        return containers;
    }
}
