/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ClassesEnhancer.java 24 2006-02-22 14:13:52Z studzine $
 * --------------------------------------------------------------------------
 */
package org.objectweb.easybeans.tests.common.enhancer;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.List;

import org.objectweb.asm.ClassReader;
import org.objectweb.asm.ClassWriter;
import org.objectweb.easybeans.deployment.annotations.analyzer.ScanClassVisitor;
import org.objectweb.easybeans.deployment.annotations.helper.ResolverHelper;
import org.objectweb.easybeans.deployment.annotations.metadata.ClassAnnotationMetadata;
import org.objectweb.easybeans.deployment.annotations.metadata.EjbJarAnnotationMetadata;
import org.objectweb.easybeans.deployment.annotations.metadata.MethodAnnotationMetadata;
import org.objectweb.easybeans.enhancer.EnhancerException;
import org.objectweb.easybeans.enhancer.interceptors.InterceptorClassAdapter;


/**
 * Enhance a set of given classes.
 * @author Florent Benoit
 */
public final class ClassesEnhancer {

    /**
     * Type available (for adding adapter).
     */
    public static enum TYPE {INTERCEPTOR, CALLBACK, ALL}

    /**
     * Class extension.
     */
    public static final String EXT_CLASS = ".class";
    /**
     * Utility class, no constructor.
     */
    private ClassesEnhancer() {

    }

    /**
     * Enhance the classes simulating a Bean.<br>
     * @param classesToEnhance the list of class on which run class adapters
     * @param type the type of adapter to run.
     * @throws Exception if it fails
     */
    public static void enhance(final List<String> classesToEnhance, final TYPE type) throws Exception {
        System.setProperty("org.apache.commons.logging.Log", "org.apache.commons.logging.impl.SimpleLog");
        System.setProperty("org.apache.commons.logging.simplelog.defaultlog", "trace");

        EjbJarAnnotationMetadata ejbJarAnnotationMetadata = new EjbJarAnnotationMetadata();
        ScanClassVisitor scanVisitor = new ScanClassVisitor(ejbJarAnnotationMetadata);
        ClassLoader loader = Thread.currentThread().getContextClassLoader();

        InputStream is = null;
        for (String clazz : classesToEnhance) {
            is = loader.getResourceAsStream(clazz);
            new ClassReader(is).accept(scanVisitor, false);
        }

        ResolverHelper.resolve(ejbJarAnnotationMetadata);

        for (ClassAnnotationMetadata classAnnotationMetadata : ejbJarAnnotationMetadata
                .getClassAnnotationMetadataCollection()) {

            if (classAnnotationMetadata.isBean()) {

                // Remove global EasyBeans interceptors
                classAnnotationMetadata.setGlobalEasyBeansInterceptors(null);
                for (MethodAnnotationMetadata m : classAnnotationMetadata.getMethodAnnotationMetadataCollection()) {
                    m.setInterceptors(null);
                }

                String className = classAnnotationMetadata.getClassName() + ".class";
                is = loader.getResourceAsStream(className);
                if (is == null) {
                    throw new EnhancerException("Cannot find input stream in classloader " + loader + " for class "
                            + className);
                }
                ClassReader cr = null;
                try {
                    cr = new ClassReader(is);
                } catch (IOException e) {
                    throw new EnhancerException("Cannot load input stream for class '" + className
                            + "' in classloader '" + loader);
                }

                ClassWriter cw = new ClassWriter(true);
                InterceptorClassAdapter cv = new InterceptorClassAdapter(classAnnotationMetadata, cw, true);
                cr.accept(cv, false);

                // define subclasses if interceptor enabled
                if (type == TYPE.INTERCEPTOR || type == TYPE.ALL) {
                    List<InterceptorClassAdapter.DefinedClass> lst = cv.getDefinedClasses();
                    if (lst != null) {
                        for (InterceptorClassAdapter.DefinedClass definedClass : lst) {
                            System.out.println("Defined class for interceptors '" + definedClass.getClassName() + "'.");
                            loadClass(definedClass.getClassName(), definedClass.getBytes());
                        }
                    }
                }

                // define class
                System.out.println("Defined class '" + classAnnotationMetadata.getClassName().replace("/", ".") + "'.");
                loadClass(classAnnotationMetadata.getClassName().replace("/", "."), cw.toByteArray());

                // check
                System.out.println("Verifing class '" + classAnnotationMetadata.getClassName());
                JCheckClassAdapter.checkClass(cw.toByteArray());

            }
        }

    }

    /**
     * Loads/defines a class in the current class loader.
     * @param className the name of the class
     * @param b the bytecode of the class to define
     */
    private static void loadClass(final String className, final byte[] b) {

        FileOutputStream fos = null;
        try {
            fos = new FileOutputStream(System.getProperty("java.io.tmpdir") + File.separator + className + ".class");
            fos.write(b);
        } catch (IOException ioe) {
            throw new RuntimeException(ioe);
        }

        // override classDefine (as it is protected) and define the class.
        try {
            ClassLoader loader = Thread.currentThread().getContextClassLoader();
            Class cls = Class.forName("java.lang.ClassLoader");
            java.lang.reflect.Method method = cls.getDeclaredMethod("defineClass", new Class[] {String.class,
                    byte[].class, int.class, int.class});

            // protected method invocaton
            method.setAccessible(true);
            try {
                Object[] args = new Object[] {className, b, new Integer(0), new Integer(b.length)};
                method.invoke(loader, args);
            } finally {
                method.setAccessible(false);
            }
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }
}
