/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: TestClassInterceptor02.java 24 2006-02-22 14:13:52Z studzine $
 * --------------------------------------------------------------------------
 */
package org.objectweb.easybeans.tests.interceptors.business;

import static org.objectweb.easybeans.tests.common.asserts.Assert.assertEquals;
import static org.objectweb.easybeans.tests.common.helper.EJBHelper.getBeanLocalInstance;

import java.util.ArrayList;
import java.util.List;

import org.objectweb.easybeans.tests.common.ejbs.stateless.ItfClassInterceptorTestRemote;
import org.objectweb.easybeans.tests.common.ejbs.stateless.SLSBClassInterceptorTest00;
import org.objectweb.easybeans.tests.common.ejbs.stateless.SLSBClassInterceptorTest02;
import org.objectweb.easybeans.tests.common.interceptors.business.PrintOrder01Interceptor;
import org.objectweb.easybeans.tests.common.interceptors.business.PrintOrder02Interceptor;
import org.objectweb.easybeans.tests.common.interceptors.business.PrintOrder03Interceptor;
import org.objectweb.easybeans.tests.common.interceptors.business.PrintOrder04Interceptor;
import org.objectweb.easybeans.tests.common.interceptors.business.PrintOrder06Interceptor;
import org.objectweb.easybeans.tests.common.interceptors.business.PrintOrder07Interceptor;
import org.objectweb.easybeans.tests.common.interceptors.business.PrintOrder08Interceptor;
import org.objectweb.easybeans.tests.common.interceptors.business.PrintOrder09Interceptor;
import org.testng.annotations.Configuration;
import org.testng.annotations.Test;

/**
 * Verifies if the order to execute the class and method interceptors is
 * correct. The correct order is:
 * <li>Default Interceptor in the order of theirs specification.</li>
 * <li>Interceptor Classes in the order of theirs specification.</li>
 * <li>Interceptor Methods in the order of theirs specification.</li>
 * <li>Interceptors annotated to a business method in the order of theirs specification.</li>
 * <br> The bean used to the test has four interceptor.
 * @reference JSR220-PROPOSED FINAL - Item:11.3.1
 * @requirement Application Server must be running; the bean
 *              org.objectweb.easybeans.tests.common.ejbs.stateless.SLSBClassInterceptorTest02.java
 *              must be deployed.
 * @setup gets the reference of SLSBClassInterceptorTest02
 * @author Gisele Pinheiro Souza
 * @author Eduardo Studzinski Estima de Castro
 */
public class TestClassInterceptor02 {

    /**
     * Bean used to implement the test.
     */
    private ItfClassInterceptorTestRemote<Integer> clBean;

    /**
     * Creates a bean to be used in the tests.
     * @throws Exception if there is a problem with the bean initialization.
     */
    @SuppressWarnings("unchecked")
    @Configuration(beforeTestMethod = true)
    public void startUp() throws Exception {
        clBean = getBeanLocalInstance(SLSBClassInterceptorTest02.class.getName());
    }

    /**
     * Verifies if the interceptors are not executed.
     * @input List with no values inside.
     * @output List with five values, the value inserted by the method and the
     *         value inserted by the interceptors.
     */
    @Test(groups = {"onlyClassInterceptor", "withInheritance"})
    public void testClassInterCallOrder00() {
        // The arrays used to know what is the interceptor order
        List<Integer> arResult = new ArrayList<Integer>();
        List<Integer> arExpected = new ArrayList<Integer>();

        // insert the value of interceptor class
        arExpected.add(PrintOrder06Interceptor.ORDER);
        // insert the value of interceptor class
        arExpected.add(PrintOrder07Interceptor.ORDER);
        // insert the value of interceptor class
        arExpected.add(PrintOrder08Interceptor.ORDER);
        // insert the value of interceptor class
        arExpected.add(PrintOrder09Interceptor.ORDER);
        // insert the value of method
        arExpected.add(SLSBClassInterceptorTest02.ORDER);

        // gets the result
        arResult = clBean.withoutMethodInterceptor(arResult);

        // compares the two values
        assertEquals(arExpected, arResult,
                "The class interceptors qre not called or they are called in the incorrect order.");
    }

    /**
     * Verifies if the interceptors are not executed.
     * @input List with no values inside.
     * @output List with five values, the value inserted by the method and the
     *         value inserted by the interceptors.
     */
    @Test(groups = {"excludeDefaultInterceptor", "onlyClassInterceptor", "withInheritance"})
    public void testClassInterCallOrder01() {
        // The arrays used to know what is the interceptor order
        List<Integer> arResult = new ArrayList<Integer>();
        List<Integer> arExpected = new ArrayList<Integer>();

        // insert the value of interceptor class
        arExpected.add(PrintOrder06Interceptor.ORDER);
        // insert the value of interceptor class
        arExpected.add(PrintOrder07Interceptor.ORDER);
        // insert the value of interceptor class
        arExpected.add(PrintOrder08Interceptor.ORDER);
        // insert the value of interceptor class
        arExpected.add(PrintOrder09Interceptor.ORDER);
        // insert the value of method
        arExpected.add(SLSBClassInterceptorTest02.ORDER);
        // gets the result
        arResult = clBean.withExcludeDefaultInterceptor(arResult);

        // compares the two values
        assertEquals(arExpected, arResult,
                "The class interceptors qre not called or they are called in the incorrect order.");
    }

    /**
     * Verifies if the interceptor classes are not executed with the annotation
     * excludeClassInterceptor.
     * @input List with no values inside.
     * @output List with only one value, the value inserted by the method.
     */
    @Test(groups = {"excludeClassInterceptor", "onlyClassInterceptor"})
    public void testClassInterCallOrder02() {
        // The arrays used to know what is the interceptor order
        List<Integer> arResult = new ArrayList<Integer>();
        List<Integer> arExpected = new ArrayList<Integer>();

        // insert the value of the method
        arExpected.add(SLSBClassInterceptorTest02.ORDER);

        // gets the result
        arResult = clBean.withExcludeClassInterceptor(arResult);

        // compares the two values
        assertEquals(arExpected, arResult,
                "The method has excludeClassInterceptor annotation, but it is calling the interceptor.");
    }

    /**
     * Verifies if the interceptor classes are not executed with the annotation
     * excludeClassInterceptor. Also, this test verify if the annotation doesn't
     * modify the interceptor method execution.
     * @input List with no values inside.
     * @output List with two values, the value inserted by the method
     *         interceptor and the value inserted by the method.
     */
    @Test(groups = {"methodInterceptor", "excludeClassInterceptor"})
    public void testClassInterCallOrder03() {
        // The arrays used to know what is the interceptor order
        List<Integer> arResult = new ArrayList<Integer>();
        List<Integer> arExpected = new ArrayList<Integer>();

        // insert the value of the interceptor
        arExpected.add(PrintOrder01Interceptor.ORDER);
        // insert the value of the method
        arExpected.add(SLSBClassInterceptorTest02.ORDER);

        // gets the result
        arResult = clBean.withExcludeClassInterAndOneMethodInter(arResult);

        // compares the two values
        assertEquals(arExpected, arResult,
                "This method has the excludeClassInterceptor, but it is not executing the method interceptor correctly.");
    }

    /**
     * Verifies if the interceptor classes are not executed with the annotation
     * excludeClassInterceptor. Also, this test verify if the annotation doesn't
     * modify many interceptor method executions.
     * @input List with no values inside.
     * @output List with two values, the value inserted by the method
     *         interceptor and the value inserted by the method.
     */
    @Test(groups = {"excludeDefaultInterceptor", "excludeClassInterceptor", "methodInterceptor", "withInheritance"})
    public void testClassInterCallOrder04() {
        // The arrays used to know what is the interceptor order
        List<Integer> arResult = new ArrayList<Integer>();
        List<Integer> arExpected = new ArrayList<Integer>();

        // insert the value of the interceptor
        arExpected.add(PrintOrder01Interceptor.ORDER);
        // insert the value of the interceptor
        arExpected.add(PrintOrder02Interceptor.ORDER);
        // insert the value of the interceptor
        arExpected.add(PrintOrder03Interceptor.ORDER);
        // insert the value of the interceptor
        arExpected.add(PrintOrder04Interceptor.ORDER);
        // insert the value of the method
        arExpected.add(SLSBClassInterceptorTest02.ORDER);

        // gets the result
        arResult = clBean.withExcludeClassDefInterAndFourMethodInter(arResult);

        // compares the two values
        assertEquals(arExpected, arResult,
                "This method has the excludeClassInterceptor, but it is not executing the method interceptors correctly.");
    }

    /**
     * Verifies if the interceptor classes and the interceptor methods work well
     * together, i.e., if they respect the order.
     * @input List with no values inside.
     * @output List with four values, the values inserted by the method
     *         interceptor and the value inserted by the method.
     */
    @Test(groups = {"methodInterceptor", "withInheritance"})
    public void testClassInterCallOrder05() {
        // The arrays used to know what is the interceptor order
        List<Integer> arResult = new ArrayList<Integer>();
        List<Integer> arExpected = new ArrayList<Integer>();

        // insert the value of interceptor class
        arExpected.add(PrintOrder06Interceptor.ORDER);
        // insert the value of interceptor class
        arExpected.add(PrintOrder07Interceptor.ORDER);
        // insert the value of interceptor class
        arExpected.add(PrintOrder08Interceptor.ORDER);
        // insert the value of interceptor class
        arExpected.add(PrintOrder09Interceptor.ORDER);
        // insert the value of the interceptor
        arExpected.add(PrintOrder01Interceptor.ORDER);
        // insert the value of the interceptor
        arExpected.add(PrintOrder02Interceptor.ORDER);
        // insert the value of the interceptor
        arExpected.add(PrintOrder03Interceptor.ORDER);
        // insert the value of the method
        arExpected.add(SLSBClassInterceptorTest02.ORDER);

        // gets the result
        arResult = clBean.withThreeMethodInterceptor(arResult);

        // compares the two values
        assertEquals(arExpected, arResult,
                "The class and the method interceptors are not running in the correct order.");
    }

    /**
     * Verifies if the interceptor classes and the interceptor methods work well
     * together, i.e., if they respect the order.
     * @input List with no values inside.
     * @output List with two values, the values inserted by the method
     *         interceptor and the value inserted by the method.
     */
    @Test(groups = {"methodInterceptor", "withInheritance"})
    public void testClassInterCallOrder06() {
        // The arrays used to know what is the interceptor order
        List<Integer> arResult = new ArrayList<Integer>();
        List<Integer> arExpected = new ArrayList<Integer>();

        // insert the value of interceptor class
        arExpected.add(PrintOrder06Interceptor.ORDER);
        // insert the value of interceptor class
        arExpected.add(PrintOrder07Interceptor.ORDER);
        // insert the value of interceptor class
        arExpected.add(PrintOrder08Interceptor.ORDER);
        // insert the value of interceptor class
        arExpected.add(PrintOrder09Interceptor.ORDER);
        // insert the value of the interceptor
        arExpected.add(PrintOrder01Interceptor.ORDER);
        // insert the value of the method
        arExpected.add(SLSBClassInterceptorTest00.ORDER);

        // gets the result
        arResult = clBean.withOneMethodInterceptor(arResult);

        // compares the two values
        assertEquals(arExpected, arResult, "The method interceptor are running in the incorrect order.");
    }
}
