/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: TestMethodInterceptor.java 24 2006-02-22 14:13:52Z studzine $
 * --------------------------------------------------------------------------
 */
package org.objectweb.easybeans.tests.interceptors.business;

import static org.objectweb.easybeans.tests.common.asserts.Assert.assertEquals;
import static org.objectweb.easybeans.tests.common.helper.EJBHelper.getBeanLocalInstance;

import java.util.ArrayList;
import java.util.List;

import org.objectweb.easybeans.tests.common.ejbs.stateless.ItfMethodInterceptorTestRemote;
import org.objectweb.easybeans.tests.common.ejbs.stateless.SLSBMethodInterceptorTest;
import org.objectweb.easybeans.tests.common.interceptors.business.PrintOrder01Interceptor;
import org.objectweb.easybeans.tests.common.interceptors.business.PrintOrder02Interceptor;
import org.objectweb.easybeans.tests.common.interceptors.business.PrintOrder03Interceptor;
import org.objectweb.easybeans.tests.common.interceptors.business.PrintOrder04Interceptor;
import org.objectweb.easybeans.tests.common.interceptors.business.PrintOrder05Interceptor;
import org.objectweb.easybeans.tests.common.interceptors.business.PrivateInterceptor;
import org.objectweb.easybeans.tests.common.interceptors.business.ProtectedInterceptor;
import org.testng.annotations.Configuration;
import org.testng.annotations.Test;

/**
 * Verifies if the order to execute the method interceptors is
 * correct. The correct order is:
 * <li>Default Interceptor in the order of theirs specification.</li>
 * <li>Interceptor Classes in the order of theirs specification.</li>
 * <li>Interceptor Methods in the order of theirs specification.</li>
 * <li>Interceptors annotated to a business method in the order of theirs specification.</li>
 * <br> The bean used to the test has one interceptor.
 * @reference JSR220-PROPOSED FINAL - Item:11.7
 * @requirement Application Server must be running; the bean
 *              org.objectweb.easybeans.tests.common.ejbs.stateless.SLSBMethodInterceptorTest.java
 *              must be deployed.
 * @setup gets the reference of SLSBMethodInterceptorTest
 * @author Gisele Pinheiro Souza
 * @author Eduardo Studzinski Estima de Castro
 */
public class TestMethodInterceptor {

    /**
     * Bean used to implement the test.
     */
    private ItfMethodInterceptorTestRemote<Integer> mtBean;

    /**
     * Creates a bean to be used in the tests.
     * @throws Exception if there is a problem with the bean initialization.
     */
    @SuppressWarnings("unchecked")
    @Configuration(beforeTestMethod = true)
    public void startUp() throws Exception {
        mtBean = getBeanLocalInstance(SLSBMethodInterceptorTest.class.getName());
    }

    /**
     * Verifies if the interceptors are not executed.
     * @input List with no values inside
     * @output List with only one value, the value inserted by the method
     */
    @Test(groups = {"withoutInterceptor"})
    public void interceptorMethodTest00() {
        // The arrays used to know what is the interceptor order
        List<Integer> arResult = new ArrayList<Integer>();
        List<Integer> arExpected = new ArrayList<Integer>();

        // insert the value of the method
        arExpected.add(SLSBMethodInterceptorTest.ORDER);

        // gets the result
        arResult = mtBean.withoutInterceptor(arResult);

        // compares the two values
        assertEquals(arExpected, arResult, "");
    }

    /**
     * Verifies if the interceptor is executed.
     * @input List with no values inside
     * @output List with two values, the value inserted by the method and the
     *         value inserted by the interceptor
     */
    @Test(groups = {"withInterceptor"})
    public void interceptorMethodTest02() {
        // The arrays used to know what is the interceptor order
        List<Integer> arResult = new ArrayList<Integer>();
        List<Integer> arExpected = new ArrayList<Integer>();

        // insert the value of the interceptor
        arExpected.add(PrintOrder01Interceptor.ORDER);
        // insert the value of the method
        arExpected.add(SLSBMethodInterceptorTest.ORDER);

        // gets the result
        arResult = mtBean.withOneMethodInterceptor(arResult);

        // compares the two values
        assertEquals(arExpected, arResult, "The interceptor is not running or it is running in the incorrect order.");
    }

    /**
     * Verifies if two interceptors are executed in order.
     * @input List with no values inside
     * @output List with three values, the value inserted by the method and the
     *         value inserted by the interceptors.
     */
    @Test(groups = {"withInterceptor"})
    public void interceptorMethodTest03() {
        // The arrays used to know what is the interceptor order
        List<Integer> arResult = new ArrayList<Integer>();
        List<Integer> arExpected = new ArrayList<Integer>();

        // insert the value of the interceptor
        arExpected.add(PrintOrder01Interceptor.ORDER);
        // insert the value of the interceptor
        arExpected.add(PrintOrder02Interceptor.ORDER);
        // insert the value of the method
        arExpected.add(SLSBMethodInterceptorTest.ORDER);

        // gets the result
        arResult = mtBean.withTwoMethodInterceptors(arResult);

        // compares the two values
        assertEquals(arExpected, arResult, "The interceptors are not called in the correct order.");
    }

    /**
     * Verifies if many interceptors are executed in order.
     * @input List with no values inside
     * @output List with six values, the value inserted by the method and the
     *         value inserted by the interceptors.
     */
    @Test(groups = {"withInterceptor", "withInheritance"})
    public void interceptorMethodTest04() {
        // The arrays used to know what is the interceptor order
        List<Integer> arResult = new ArrayList<Integer>();
        List<Integer> arExpected = new ArrayList<Integer>();

        // insert the value of the interceptor
        arExpected.add(PrintOrder01Interceptor.ORDER);
        // insert the value of the interceptor
        arExpected.add(PrintOrder02Interceptor.ORDER);
        // insert the value of the method
        arExpected.add(PrintOrder03Interceptor.ORDER);
        // insert the value of the interceptor
        arExpected.add(PrintOrder04Interceptor.ORDER);
        // insert the value of the interceptor
        arExpected.add(PrintOrder05Interceptor.ORDER);
        // insert the value of the method
        arExpected.add(SLSBMethodInterceptorTest.ORDER);

        // gets the result
        arResult = mtBean.withFiveMethodInterceptors(arResult);

        // compares the two values
        assertEquals(arExpected, arResult, "The interceptors are not running in the correct order."
                + " Maybe there is a problem with the interceptors inheritance.");
    }

    /**
     * Verifies if many interceptors are executed in order. This must repect the
     * declaration order.
     * @input List with no values inside
     * @output List with six values, the value inserted by the method and the
     *         value inserted by the interceptors.
     */
    @Test(groups = {"withInterceptor", "withInheritance"})
    public void interceptorMethodTest05() {
        // The arrays used to know what is the interceptor order
        List<Integer> arResult = new ArrayList<Integer>();
        List<Integer> arExpected = new ArrayList<Integer>();

        // insert the value of the interceptor
        arExpected.add(PrintOrder05Interceptor.ORDER);
        // insert the value of the interceptor
        arExpected.add(PrintOrder04Interceptor.ORDER);
        // insert the value of the method
        arExpected.add(PrintOrder03Interceptor.ORDER);
        // insert the value of the interceptor
        arExpected.add(PrintOrder02Interceptor.ORDER);
        // insert the value of the interceptor
        arExpected.add(PrintOrder01Interceptor.ORDER);
        // insert the value of the method
        arExpected.add(SLSBMethodInterceptorTest.ORDER);

        // gets the result
        arResult = mtBean.withFiveMethodInterceptorsInverse(arResult);

        // compares the two values
        assertEquals(arExpected, arResult, "The interceptors are not running in the correct order."
                + " Maybe there is a problem with the interceptors inheritance.");
    }

    /**
     * Verifies if interceptors are executed in order. All interceptors have a
     * private method that intercepts.
     * @input List with no values inside
     * @output List with 4 values, the value inserted by the method and the
     *         value inserted by the interceptors.
     */
    @Test(groups = {"withInterceptor"})
    public void interceptorMethodTest06() {
        // The arrays used to know what is the interceptor order
        List<Integer> arResult = new ArrayList<Integer>();
        List<Integer> arExpected = new ArrayList<Integer>();

        // insert the value of the interceptor
        arExpected.add(PrivateInterceptor.ORDER);
        // insert the value of the interceptor
        arExpected.add(PrivateInterceptor.ORDER);
        // insert the value of the interceptor
        arExpected.add(PrivateInterceptor.ORDER);
        // insert the value of the method
        arExpected.add(SLSBMethodInterceptorTest.ORDER);

        // gets the result
        arResult = mtBean.withPrivateInterceptors(arResult);

        // compares the two values
        assertEquals(arExpected, arResult, "The interceptors with private method are not running in the correct order.");
    }

    /**
     * Verifies if interceptors are executed in order. All interceptors have a
     * protected method that intercepts.
     * @input List with no values inside
     * @output List with 3 values, the value inserted by the method and the
     *         value inserted by the interceptors.
     */
    @Test(groups = {"withInterceptor"})
    public void interceptorMethodTest07() {
        // The arrays used to know what is the interceptor order
        List<Integer> arResult = new ArrayList<Integer>();
        List<Integer> arExpected = new ArrayList<Integer>();

        // insert the value of the interceptor
        arExpected.add(ProtectedInterceptor.ORDER);
        // insert the value of the interceptor
        arExpected.add(ProtectedInterceptor.ORDER);
        // insert the value of the method
        arExpected.add(SLSBMethodInterceptorTest.ORDER);

        // gets the result
        arResult = mtBean.withProtectedInterceptors(arResult);

        // compares the two values
        assertEquals(arExpected, arResult,
                "The interceptors with protected method are not running in the correct order.");
    }

    /**
     * Verifies if interceptors are executed in order. There is interceptors
     * with private, protected, and public method, also there is inheritance in
     * one interceptor(PrintOrder03Interceptor.class).
     * @input List with no values inside
     * @output List with 9 values, the value inserted by the method and the
     *         value inserted by the interceptors.
     */
    @Test(groups = {"withInterceptor", "withInheritance"})
    public void interceptorMethodTest08() {
        // The arrays used to know what is the interceptor order
        List<Integer> arResult = new ArrayList<Integer>();
        List<Integer> arExpected = new ArrayList<Integer>();

        // insert the value of the interceptor
        arExpected.add(PrivateInterceptor.ORDER);
        // insert the value of the interceptor
        arExpected.add(ProtectedInterceptor.ORDER);
        // insert the value of the interceptor
        arExpected.add(PrintOrder01Interceptor.ORDER);
        // insert the value of the interceptor
        arExpected.add(PrivateInterceptor.ORDER);
        // insert the value of the interceptor
        arExpected.add(PrintOrder03Interceptor.ORDER);
        // insert the value of the interceptor
        arExpected.add(PrivateInterceptor.ORDER);
        // insert the value of the interceptor
        arExpected.add(ProtectedInterceptor.ORDER);
        // insert the value of the interceptor
        arExpected.add(ProtectedInterceptor.ORDER);
        // insert the value of the method
        arExpected.add(SLSBMethodInterceptorTest.ORDER);

        // gets the result
        arResult = mtBean.withPrivateProtectedPublicInterceptors(arResult);

        // compares the two values
        assertEquals(arExpected, arResult, "The interceptors are not running in the correct order.");
    }
}
