/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: TestMiscInterceptor00.java 24 2006-02-22 14:13:52Z studzine $
 * --------------------------------------------------------------------------
 */
package org.objectweb.easybeans.tests.interceptors.business;

import static org.objectweb.easybeans.tests.common.asserts.Assert.assertEquals;
import static org.objectweb.easybeans.tests.common.helper.EJBHelper.getBeanLocalInstance;

import java.util.ArrayList;
import java.util.List;

import org.objectweb.easybeans.tests.common.ejbs.stateless.ItfSimpleBeanRemote;
import org.objectweb.easybeans.tests.common.ejbs.stateless.SLSBSimpleInterceptorTest00;
import org.objectweb.easybeans.tests.common.ejbs.stateless.SLSBSimpleInterceptorTest01;
import org.objectweb.easybeans.tests.common.ejbs.stateless.SLSBSimpleInterceptorTest02;
import org.objectweb.easybeans.tests.common.interceptors.business.PrintOrder01Interceptor;
import org.objectweb.easybeans.tests.common.interceptors.business.PrintOrder03Interceptor;
import org.objectweb.easybeans.tests.common.interceptors.business.PrintOrder06Interceptor;
import org.testng.annotations.Configuration;
import org.testng.annotations.Test;

/**
 * Verify if the order to execute the method and callbacks interceptors is
 * correct, also verify the interceptors inheritance. The correct order is:
 * <li>Default Interceptor in the order of theirs specification.</li>
 * <li>Interceptor Classes in the order of theirs specification.</li>
 * <li>Interceptor Methods in the order of theirs specification.</li>
 * <li>Interceptors annotated to a business method in the order of theirs specification.</li>
 * <br> The bean used to the test has one interceptor.
 * @reference JSR220-PROPOSED FINAL - Item: 11.3.1
 * @requirement Application Server must be running; the bean
 *              org.objectweb.easybeans.tests.common.ejbs.stateless.SLSBSimpleInterceptorTest.java
 *              must be deployed.
 * @setup gets the reference of SLSBSimpleInterceptorTest00,
 *        SLSBSimpleInterceptorTest01, SLSBSimpleInterceptorTest02
 * @author Eduardo Studzinski Estima de Castro
 * @author Gisele Pinheiro Souza
 */
public class TestMiscInterceptor00 {

    /**
     * Bean used to implement the test.
     */
    private ItfSimpleBeanRemote<Integer> mtBean00;

    /**
     * Bean used to implement the test.
     */
    private ItfSimpleBeanRemote<Integer> mtBean01;

    /**
     * Bean used to implement the test.
     */
    private ItfSimpleBeanRemote<Integer> mtBean02;

    /**
     * Creates beans to be used in the tests.
     * @throws Exception if there is a problem with the bean initialization.
     */
    @SuppressWarnings("unchecked")
    @Configuration(beforeTestMethod = true)
    public void startUp() throws Exception{
        mtBean00 = getBeanLocalInstance(SLSBSimpleInterceptorTest00.class.getName());
        mtBean01 = getBeanLocalInstance(SLSBSimpleInterceptorTest01.class.getName());
        mtBean02 = getBeanLocalInstance(SLSBSimpleInterceptorTest02.class.getName());
    }

    /**
     * Verifies if the interceptors are following the order of declaration.
     * There is an interceptor inside the bean class that has public modifier
     * access.
     * @input List with no values inside
     * @output List with 4 values, the value inserted by the method and value
     *         inserted by the interceptors.
     */
    @Test(groups = {"withInterceptor"})
    public void testMiscInterCallOrder00() {
        // The arrays used to know what is the interceptor order
        List<Integer> arResult = new ArrayList<Integer>();
        List<Integer> arExpected = new ArrayList<Integer>();

        // insert the value of interceptor
        arExpected.add(PrintOrder06Interceptor.ORDER);
        // insert the value of interceptor
        arExpected.add(ItfSimpleBeanRemote.EMBEDDED_INTERCEPTOR);
        // insert the value of interceptor
        arExpected.add(PrintOrder01Interceptor.ORDER);
        // insert the value of the method
        arExpected.add(SLSBSimpleInterceptorTest00.ORDER);

        // gets the result
        arResult = mtBean00.withInterceptors(arResult);

        assertEquals(arExpected, arResult,
                "The method has class interceptors, a method interceptor and a public bean interceptor method"
                        + " declared inside the bean; however it isn't running in the correct order.");
    }

    /**
     * Verifies if the interceptors are following the order of declaration.
     * There is an interceptor inside the bean class that has protected modifier
     * access.
     * @input List with no values inside
     * @output List with 4 values, the value inserted by the method and value
     *         inserted by the interceptors.
     */
    @Test(groups = {"withInterceptor"})
    public void testMiscPublicInterCallOrder01() {
        // The arrays used to know what is the interceptor order
        List<Integer> arResult = new ArrayList<Integer>();
        List<Integer> arExpected = new ArrayList<Integer>();

        // insert the value of interceptor
        arExpected.add(PrintOrder06Interceptor.ORDER);
        // insert the value of interceptor
        arExpected.add(ItfSimpleBeanRemote.EMBEDDED_INTERCEPTOR);
        // insert the value of interceptor
        arExpected.add(PrintOrder01Interceptor.ORDER);
        // insert the value of the method
        arExpected.add(SLSBSimpleInterceptorTest01.ORDER);

        // gets the result
        arResult = mtBean01.withInterceptors(arResult);

        // compares the two values
        assertEquals(arExpected, arResult,
                "The method has class interceptors, a method interceptor and a protected bean interceptor method"
                        + " declared inside the bean; however it isn't running in the correct order.");
    }

    /**
     * Verifies if the interceptors are following the order of declaration.
     * There is an interceptor inside the bean class that has private modifier
     * access.
     * @input List with no values inside
     * @output List with 4 values, the value inserted by the method and value
     *         inserted by the interceptors.
     */
    @Test(groups = {"withInterceptor"})
    public void testMiscPrivateInterCallOrder02() {
        // The arrays used to know what is the interceptor order
        List<Integer> arResult = new ArrayList<Integer>();
        List<Integer> arExpected = new ArrayList<Integer>();

        // insert the value of interceptor
        arExpected.add(PrintOrder06Interceptor.ORDER);
        // insert the value of interceptor
        arExpected.add(ItfSimpleBeanRemote.EMBEDDED_INTERCEPTOR);
        // insert the value of interceptor
        arExpected.add(PrintOrder01Interceptor.ORDER);
        // insert the value of the method
        arExpected.add(SLSBSimpleInterceptorTest02.ORDER);

        // gets the result
        arResult = mtBean02.withInterceptors(arResult);

        // compares the two values
        assertEquals(arExpected, arResult,
                "The method has class interceptors, a method interceptor and a protected bean interceptor method"
                        + " declared inside the bean; however it isn't running in the correct order.");
    }

    /**
     * Verifies if the interceptors are following the order of declaration.
     * There is an interceptor inside the bean class that has public modifier
     * access.
     * @input List with no values inside
     * @output List with 4 values, the value inserted by the method and value
     *         inserted by the interceptors.
     */
    @Test(groups = {"withInterceptor", "withInheritance"})
    public void testMiscInheritanceInterCallOrder00() {
        // The arrays used to know what is the interceptor order
        List<Integer> arResult = new ArrayList<Integer>();
        List<Integer> arExpected = new ArrayList<Integer>();

        // insert the value of interceptor
        arExpected.add(PrintOrder06Interceptor.ORDER);
        // insert the value of interceptor
        arExpected.add(ItfSimpleBeanRemote.EMBEDDED_INTERCEPTOR);
        // insert the value of interceptor
        arExpected.add(PrintOrder03Interceptor.ORDER);
        // insert the value of the method
        arExpected.add(SLSBSimpleInterceptorTest00.ORDER);

        // gets the result
        arResult = mtBean00.withInterceptorsInheritance(arResult);

        // compares the two values
        assertEquals(arExpected, arResult,
                "The method has class interceptors, a method interceptor with inheritance and a public bean interceptor method"
                        + " declared inside the bean; however it isn't running in the correct order.");
    }

    /**
     * Verifies if the interceptors are following the order of declaration.
     * There is an interceptor inside the bean class that has protected modifier
     * access.
     * @input List with no values inside
     * @output List with 4 values, the value inserted by the method and value
     *         inserted by the interceptors.
     */
    @Test(groups = {"withInterceptor", "withInheritance"})
    public void testMiscInheritanceInterCallOrder01() {
        // The arrays used to know what is the interceptor order
        List<Integer> arResult = new ArrayList<Integer>();
        List<Integer> arExpected = new ArrayList<Integer>();

        // insert the value of interceptor
        arExpected.add(PrintOrder06Interceptor.ORDER);
        // insert the value of interceptor
        arExpected.add(ItfSimpleBeanRemote.EMBEDDED_INTERCEPTOR);
        // insert the value of interceptor
        arExpected.add(PrintOrder03Interceptor.ORDER);
        // insert the value of the method
        arExpected.add(SLSBSimpleInterceptorTest01.ORDER);

        // gets the result
        arResult = mtBean01.withInterceptorsInheritance(arResult);

        // compares the two values
        assertEquals(arExpected, arResult,
                "The method has class interceptors, a method interceptor with inheritance and a protected bean interceptor method"
                        + " declared inside the bean; however it isn't running in the correct order.");
    }

    /**
     * Verifies if the interceptors are following the order of declaration.
     * There is an interceptor inside the bean class that has private modifier
     * access.
     * @input List with no values inside
     * @output List with 4 values, the value inserted by the method and value
     *         inserted by the interceptors.
     */
    @Test(groups = {"withInterceptor", "withInheritance"})
    public void testMiscInheritanceInterCallOrder02() {
        // The arrays used to know what is the interceptor order
        List<Integer> arResult = new ArrayList<Integer>();
        List<Integer> arExpected = new ArrayList<Integer>();

        // insert the value of interceptor
        arExpected.add(PrintOrder06Interceptor.ORDER);
        // insert the value of interceptor
        arExpected.add(ItfSimpleBeanRemote.EMBEDDED_INTERCEPTOR);
        // insert the value of interceptor
        arExpected.add(PrintOrder03Interceptor.ORDER);
        // insert the value of the method
        arExpected.add(SLSBSimpleInterceptorTest02.ORDER);

        // gets the result
        arResult = mtBean02.withInterceptorsInheritance(arResult);

        // compares the two values
        assertEquals(arExpected, arResult,
                "The method has class interceptors, a method interceptor with inheritance and a private bean interceptor method"
                        + " declared inside the bean; however it isn't running in the correct order.");
    }
}
